//
// Collection of optional macros to be used in GUIMainMacro
//
// This macro file is loaded by the function, LoadLibraries(), defined in 
// the file, $JSFROOT/macro/LoadLibraries.C, during the construction of 
// JSFSteer object.  
// The default file name of this macro file is defined by the JSFEnv varibale, 
// JSFJ4.S4MacroFileName.  The default value is $SATELLITES/macro/S4Macro.C
//
//$Id: S4Macros.C,v 1.18 2008/08/22 00:57:06 miyamoto Exp $

Bool_t gS4Macros_hasMakers=kFALSE;

// ----------------------------------------------------------------------
void LoadS4Libraries(JSFEnv *env=0)
{
//  This function is called during the construction of JSFSteer object,
//  through the function, LoadLibraries(), defined in $JSFROOT/macro/LoadLibraries.C

  if( env->GetValue("JSFGUI.SimulationType",1) == 4 ) {
    S4LibraryLoader("JSFFromLCIO",env);
    std::cerr << " libJSFFromLCIO.so was loaded" << std::endl;
  }
  else if( env->GetValue("JSFGUI.SimulationType",1) == 3 ) {
    S4LibraryLoader("JSFJupiter",env);
    std::cerr << " libJSFJupiter.so was loaded" << std::endl;
  }

  //  S4LibraryLoader("mctruths",env);


// JSFGUI.S4Sequence: mctruths:lcio:makers:analysis
  TString inputs(env->GetValue("JSFGUI.S4Sequence",""));
  TString delim(":");
  TObjArray *tokens=inputs.Tokenize(delim);
  if( tokens->GetEntries() < 1 ) return;
  TIter next(tokens);
  TObjString *objstr=0;
  while( (objstr=(TObjString*)next()) ) {
    TString setname=objstr->GetString();
    if( setname != TString("mctruths") ) S4LibraryLoader(setname, env);
  }

}

// ----------------------------------------------------------------------
void ConstructS4Modules()
{
// This function is called by the function, DefineOptionalModules(), 
// which is defined in $JSFROOT/macro/OptFunctions.C. 

// JSFGUI.S4Sequence: mctruths:lcio:makers:analysis
  TString inputs(gJSF->Env()->GetValue("JSFGUI.S4Sequence",""));
  TString delim(":");
  TObjArray *tokens=inputs.Tokenize(delim);
  if( tokens->GetEntries() < 1 ) return;
  TIter next(tokens);
  TObjString *objstr=0;
  while( (objstr=(TObjString*)next()) ) {
    TString setname=objstr->GetString();
    S4ModuleDefine(setname);
  }

}

// ----------------------------------------------------------------------
void S4Initialize()
{
 // Called during the job initialization to perform the initialization 
 // task for SATELLITES.  Called from the function, Initialize(),
 // defined in $JSFROOT/macro/GUIMainMacro.C, if 
 

//----------------------------------
// set parameters
//----------------------------------

  if( gS4Macros_hasMakers ) {
 
    std::cerr << "===============================================" << std::endl;
    std::cerr << " Setting Satellite parameters in S4Initialize() of "
	      << " S4Macros.C " << std::endl;
    std::cerr << "===============================================" << std::endl;

   // ===============================================
   Double_t bf=gJSF->Env()->GetValue("J4SOL.BField",3.5);
   U4VKalDetector::SetBfieldTesla(bf);
   std::cerr << "Solenoid field for U4VKalDetector is set to " << bf << "(Tesla)" << std::endl; 

   // ===============================================
   S4VTXHit::SetSigRPhi(gJSF->Env()->GetValue("S4VTX.SigRPhi",2.8e-4));
   S4VTXHit::SetSigZ(gJSF->Env()->GetValue("S4VTX.SigZ",2.8e-4));
   S4VTXHit::Print();

   // ===============================================
   S4ITHit::SetSigRPhi(gJSF->Env()->GetValue("S4IT.SigRPhi",10.0e-4));
   S4ITHit::SetSigZ(gJSF->Env()->GetValue("S4IT.SigZ",10.0e-4));
   S4ITHit::SetSigX(gJSF->Env()->GetValue("S4IT.SigX",10.0e-4));
   S4ITHit::SetSigY(gJSF->Env()->GetValue("S4IT.SigY",10.0e-4));
   S4ITHit::Print();

    // See $SATELLITES/src/metis/tpc/hitmaker/S4TPCHit.cc 
    // for the meenings of these parameters.
   TString tpcres=TString(gJSF->Env()->GetValue("S4TPC.SpatialResolution","ILDAuto"));
   if( tpcres == "ILD" || tpcres == "ILDAuto" || tpcres == "ILCAuto_bsq") {
     S4TPCHit::SetSpatialResolution("ILD");
     S4TPCHit::SetSig00(gJSF->Env()->GetValue("S4TPC.Sig00",0.005));
     S4TPCHit::SetSig0SinDep(gJSF->Env()->GetValue("S4TPC.Sig0SinDep",0.090));
     Double_t sigldep=gJSF->Env()->GetValue("S4TPC.SigLDep",0.00053);
     if( tpcres == "ILDAuto" ) {
       Double_t bf=gJSF->Env()->GetValue("J4SOL.BField",3.5);
       sigldep*=4.0/bf;
       std::cerr << "    SpatioalResolution is ILDAuto : "
          << " Cd^2/Neff term is scaled by BField.  B=" << bf << " Tesla " << std::endl; 
       std::cerr << " Scaled value of Cd/sqrt(Neff) =" << sigldep << " um " << std::endl;
       S4TPCHit::SetSigLDep(sigldep);
     }  
     if( tpcres == "ILDAuto_bsq" ) {
       Double_t bf=gJSF->Env()->GetValue("J4SOL.BField",3.5);
       sigldep*=4.0*4.0/bf/bf;
       std::cerr << "    SpatioalResolution is ILDAuto : "
          << " Cd^2/Neff term is scaled by BField.  B=" << bf << " Tesla " << std::endl; 
       std::cerr << " Scaled value of Cd/sqrt(Neff) =" << sigldep << " um " << std::endl;
       S4TPCHit::SetSigLDep(sigldep);
     }  
   }
   //
   else if ( tpcres == "experimental" ) {
     S4TPCHit::SetSigma0(gJSF->Env()->GetValue("S4TPC.Sigma0",5.5e-3));
     S4TPCHit::SetCd(gJSF->Env()->GetValue("S4TPC.Cd",5.533e-3));
     S4TPCHit::SetAlpha(gJSF->Env()->GetValue("S4TPC.Alpha",0.4444));
     S4TPCHit::SetNe(gJSF->Env()->GetValue("S4TPC.Ne",63));
   }
   else if ( tpcres == "flat" ) {
     S4TPCHit::SetSigmaRPhi(gJSF->Env()->GetValue("S4TPC.DRPhi",0.015));
   }
   S4TPCHit::SetSigmaRPhi(gJSF->Env()->GetValue("S4TPC.SigmaZ",0.0400));
   S4TPCHit::Print();

   // ===============================================

   S4CALHitBuf::SetHitMakingScheme(
   gJSF->Env()->GetValue("JSFJ4.CALHitMakingScheme",1)); 
	// idealistic (smeared 1 hit/1 exact hit)

   //   S4CALCluster::SetCalibFactorForEM(37.0142);  // for mar06_2m, tower data
   //   S4CALCluster::SetCalibFactorForHD(35.2893);

   S4CALCluster::SetCalibFactorForEM(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrFactorForEM",24.3367));
   S4CALCluster::SetCalibFactorForHD(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrFactorForHD",33.5625));


   S4CALCluster::SetCalibBERatioForEM(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrBERatioForEM",-1.0));
   S4CALCluster::SetCalibBERatioForHD(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrBERatioForHD",-1.0));

   S4CALCluster::PrintConstants();
 
//----------------------------------
// set subtrackers to hybtrackmaker
//----------------------------------
 
  S4VTXTrackMaker *vtxtrackmaker = (S4VTXTrackMaker *)gJSF->FindModule("S4VTXTrackMaker");
  S4ITTrackMaker  *ittrackmaker  = (S4ITTrackMaker *)gJSF->FindModule("S4ITTrackMaker");
  S4TPCTrackMaker *tpctrackmaker = (S4TPCTrackMaker *)gJSF->FindModule("S4TPCTrackMaker");
  S4HYBTrackMaker *htrackmaker   = (S4HYBTrackMaker *)gJSF->FindModule("S4HYBTrackMaker");

  TKalDetCradle    *cradle     = new TKalDetCradle();
  U4KalDetector    *det        = new U4KalDetector();
  U4TPCKalDetector *tpcdet     = new U4TPCKalDetector("FillLayer");
  U4ITKalDetector  *itdet      = new U4ITKalDetector("FillLayer");
  U4VTXKalDetector *vtxdet     = new U4VTXKalDetector("FillLayer");

  tpctrackmaker->SetDetector(*tpcdet);
  ittrackmaker ->SetDetector(*itdet);
  vtxtrackmaker->SetDetector(*vtxdet);
  htrackmaker->SetDetector(*det);

  cradle->Install(*det);
  cradle->Install(*tpcdet);
  cradle->Install(*itdet);
  cradle->Install(*vtxdet);
  cradle->Close();

  htrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);

    std::cerr << "===============================================" << std::endl;
    std::cerr << "===  End of S4Initialize                   ====" << std::endl;
    std::cerr << "===============================================" << std::endl;
  }

}

// ----------------------------------------------------------------------
void S4LibraryLoader(TString setname, JSFEnv *env)
{
  cerr  << "Loading libraries for set " << setname << endl;
  if( setname == TString("lcio") ) {
    if ( env->GetValue("JSFJ4.CALHitMakingScheme",1) == 0 ) {
      LoadLibraryWithMessage("libS4Kalman.so");
      LoadLibraryWithMessage("libS4Geom.so");
      LoadLibraryWithMessage("libS4KalTrack.so");
      LoadLibraryWithMessage("libU4VHitMaker.so");
      LoadLibraryWithMessage("libU4VTrackMaker.so");
      LoadLibraryWithMessage("libS4Kern.so");
      LoadLibraryWithMessage("libS4CALHitMaker.so");
    }
    LoadLibraryWithMessage("libJSF2LCIO.so");
    LoadLibraryWithMessage("libJ42LCIO.so");
  }
  else if ( setname == TString("mctruths") ) { 
    LoadLibraryWithMessage("libS4MCTruth.so");
    LoadLibraryWithMessage("libS4GENMCTruth.so");
    LoadLibraryWithMessage("libS4HISTMCTruth.so");
    LoadLibraryWithMessage("libS4VTXMCTruth.so");
    LoadLibraryWithMessage("libS4ITMCTruth.so");
    LoadLibraryWithMessage("libS4TPCMCTruth.so");
    LoadLibraryWithMessage("libS4CDCMCTruth.so");
    LoadLibraryWithMessage("libS4CALMCTruth.so");
    LoadLibraryWithMessage("libS4MUDMCTruth.so");
    LoadLibraryWithMessage("libS4FCALMCTruth.so");
    LoadLibraryWithMessage("libS4BCALMCTruth.so");
  }
  else if ( setname == TString("JSFFromLCIO") ) { 
    LoadLibraryWithMessage("libS4Utils.so");
    LoadLibraryWithMessage("libJSFJ4.so");
    LoadLibraryWithMessage("libS4MCTruth.so");
    LoadLibraryWithMessage("libS4GENMCTruth.so");
    LoadLibraryWithMessage("libS4VTXMCTruth.so");
    LoadLibraryWithMessage("libS4ITMCTruth.so");
    LoadLibraryWithMessage("libS4TPCMCTruth.so");
    LoadLibraryWithMessage("libS4CALMCTruth.so");
    LoadLibraryWithMessage("libS4MUDMCTruth.so");
    LoadLibraryWithMessage("libS4FCALMCTruth.so");
    LoadLibraryWithMessage("libS4BCALMCTruth.so");
    LoadLibraryWithMessage("libS4CDCMCTruth.so");
    LoadLibraryWithMessage("libS4CTMCTruth.so");
    LoadLibraryWithMessage("libJSFFromLCIO.so");
  }
  else if ( setname == TString("JSFJupiter") ) { 
    LoadLibraryWithMessage("libS4Utils.so");
    LoadLibraryWithMessage("libJSFJ4.so");
    LoadLibraryWithMessage("libS4MCTruth.so");
    LoadLibraryWithMessage("libS4GENMCTruth.so");
    LoadLibraryWithMessage("libS4VTXMCTruth.so");
    LoadLibraryWithMessage("libS4ITMCTruth.so");
    LoadLibraryWithMessage("libS4TPCMCTruth.so");
    LoadLibraryWithMessage("libS4CALMCTruth.so");
    LoadLibraryWithMessage("libS4MUDMCTruth.so");
    LoadLibraryWithMessage("libS4FCALMCTruth.so");
    LoadLibraryWithMessage("libS4BCALMCTruth.so");
    LoadLibraryWithMessage("libS4CDCMCTruth.so");
    LoadLibraryWithMessage("libS4CTMCTruth.so");

    LoadLibraryWithMessage("libS4HISTMCTruth.so");
    LoadLibraryWithMessage("libJSFJupiter.so");
  }
  else if ( setname == TString("makers") ) {
    LoadLibraryWithMessage("libS4Kalman.so");
    LoadLibraryWithMessage("libS4Geom.so");
    LoadLibraryWithMessage("libS4KalTrack.so");
    LoadLibraryWithMessage("libU4VHitMaker.so");
    LoadLibraryWithMessage("libU4KalDetector.so");
    LoadLibraryWithMessage("libU4VTXKalDetector.so");
    LoadLibraryWithMessage("libU4ITKalDetector.so");
    LoadLibraryWithMessage("libU4TPCKalDetector.so");

    LoadLibraryWithMessage("libU4TPCHitMaker.so");
    LoadLibraryWithMessage("libS4TPCHitMaker.so");
    LoadLibraryWithMessage("libU4ITHitMaker.so");
    LoadLibraryWithMessage("libS4ITHitMaker.so");
    LoadLibraryWithMessage("libU4VTXHitMaker.so");
    LoadLibraryWithMessage("libS4VTXHitMaker.so");

    LoadLibraryWithMessage("libU4VTrackMaker.so");
    LoadLibraryWithMessage("libS4Kern.so");
    LoadLibraryWithMessage("libU4TPCTrackMaker.so");
    LoadLibraryWithMessage("libS4TPCTrackMaker.so");
    LoadLibraryWithMessage("libU4ITTrackMaker.so");
    LoadLibraryWithMessage("libS4ITTrackMaker.so");
    LoadLibraryWithMessage("libU4VTXTrackMaker.so");
    LoadLibraryWithMessage("libS4VTXTrackMaker.so");
    LoadLibraryWithMessage("libU4HYBTrackMaker.so");
    LoadLibraryWithMessage("libS4HYBTrackMaker.so");

    LoadLibraryWithMessage("libS4CALHitMaker.so");
    LoadLibraryWithMessage("libS4CALClusterMaker.so");

    LoadLibraryWithMessage("libAnlib.so");
    LoadLibraryWithMessage("libS4PFOMaker.so");
    LoadLibraryWithMessage("libS4JetMaker.so");
    gS4Macros_hasMakers=kTRUE;

  }
  else if ( setname == TString("analysis") ) {
    LoadLibraryWithMessage("libS4TPCAnalysis.so");
    LoadLibraryWithMessage("libS4ITAnalysis.so");
    LoadLibraryWithMessage("libS4VTXAnalysis.so");
    LoadLibraryWithMessage("libS4CALAnalysis.so");
    LoadLibraryWithMessage("libS4PFOAnalysis.so");
    LoadLibraryWithMessage("libS4JetAnalysis.so");
  }
  else {
    cerr << "Undefined library set name " << setname << " was used in S4LibraryLoader() " << endl;
    gSystem->Exit(-1);
  }

}

// ----------------------------------------------------------------------
void S4ModuleDefine(TString setname)
{
  cerr << " Define modules for set " << setname << endl;

  if( setname == TString("lcio") ) {
    if( gJSF->Env()->GetValue("JSFJ4.CALHitMakingScheme",1) == 0 ) {
      S4CALHitMaker     *calhitmaker   = new S4CALHitMaker();
      calhitmaker   -> SetMakeBranch(kFALSE);
    }
    J42LCIO *j42lcio=new J42LCIO();
    j42lcio -> SetMakeBranch(kFALSE);
  }
  else if ( setname == TString("mctruths") ) {
    JSFJ4BufManager *bufman  = new JSFJ4BufManager();  
    S4GENMCTruth  *genmctruth   = new S4GENMCTruth();  
    S4HISTMCTruth *histmctruth  = new S4HISTMCTruth();  

    bufman     -> SetMakeBranch(kFALSE);
    genmctruth -> SetMakeBranch(kFALSE);
    if( gJSF->Env()->GetValue("JSFJ4.HasVTX",1) ){
      S4VTXMCTruth  *vtxmctruth   = new S4VTXMCTruth();  
      vtxmctruth -> SetMakeBranch(kFALSE);
    }
    if( gJSF->Env()->GetValue("JSFJ4.HasIT",1) ){
      S4ITMCTruth   *itmctruth    = new S4ITMCTruth();   
      itmctruth  -> SetMakeBranch(kFALSE);
    }
    if( gJSF->Env()->GetValue("JSFJ4.HasTPC",1) ){
      S4TPCMCTruth  *tpcmctruth   = new S4TPCMCTruth();  
      tpcmctruth -> SetMakeBranch(kFALSE);
    }
    if( gJSF->Env()->GetValue("JSFJ4.HasCLX",1) == 1 ||
	gJSF->Env()->GetValue("JSFJ4.HasCAL",1) == 1 ) {
      S4CALMCTruth  *calmctruth   = new S4CALMCTruth();  
      calmctruth -> SetMakeBranch(kFALSE);
    }
    if( gJSF->Env()->GetValue("JSFJ4.HasMUD",1) == 1 ) {
      S4MUDMCTruth  *mudmctruth   = new S4MUDMCTruth();  
      mudmctruth -> SetMakeBranch(kFALSE);
    }
    if( gJSF->Env()->GetValue("JSFJ4.HasIR",1) == 1 ) {
      S4FCALMCTruth *fcalmctruth  = new S4FCALMCTruth();
      S4BCALMCTruth *bcalmctruth  = new S4BCALMCTruth();
      bcalmctruth -> SetMakeBranch(kFALSE);
      fcalmctruth -> SetMakeBranch(kFALSE);
    }
    histmctruth -> SetMakeBranch(kFALSE);

  }
  else if ( setname == TString("makers") ) {
//----------------------------------
// Define KalDetector
//----------------------------------
     S4TPCHitMaker     *tpchitmaker   = new S4TPCHitMaker();   
     S4TPCTrackMaker   *tpctrackmaker = new S4TPCTrackMaker();
     S4ITHitMaker      *ithitmaker    = new S4ITHitMaker();
     S4ITTrackMaker    *ittrackmaker  = new S4ITTrackMaker();
     S4VTXHitMaker     *vtxhitmaker   = new S4VTXHitMaker();
     S4VTXTrackMaker   *vtxtrackmaker = new S4VTXTrackMaker();
     S4HYBTrackMaker   *htrackmaker   = new S4HYBTrackMaker();
                                                                                
     S4CALHitMaker     *calhitmaker   = new S4CALHitMaker();
     S4CALClusterMaker *calclustmaker = new S4CALClusterMaker();
     S4PFOMaker        *pfomaker      = new S4PFOMaker();
     S4JetMaker        *jetmaker      = new S4JetMaker();
                                                                                
     tpchitmaker   -> SetMakeBranch(kFALSE);
     tpctrackmaker -> SetMakeBranch(kFALSE);
     ithitmaker    -> SetMakeBranch(kFALSE);
     ittrackmaker  -> SetMakeBranch(kFALSE);
     vtxhitmaker    -> SetMakeBranch(kFALSE);
     vtxtrackmaker  -> SetMakeBranch(kFALSE);
     htrackmaker   -> SetMakeBranch(kFALSE);
                                                                                
     calhitmaker   -> SetMakeBranch(kFALSE);
     calclustmaker -> SetMakeBranch(kFALSE);
     pfomaker      -> SetMakeBranch(kFALSE);
     jetmaker      -> SetMakeBranch(kFALSE);
                                                                                
   }
   else if ( setname == TString("analysis") ) {
     S4TPCAnalysis     *tpcanalysis   = new S4TPCAnalysis();
     S4ITAnalysis      *itanalysis    = new S4ITAnalysis();
     S4VTXAnalysis     *vtxanalysis   = new S4VTXAnalysis();
     S4CALAnalysis     *calanalysis   = new S4CALAnalysis();
     S4PFOAnalysis     *pfoanalysis   = new S4PFOAnalysis();
     S4JetAnalysis     *jetanalysis   = new S4JetAnalysis();

     tpcanalysis   -> SetMakeBranch(kFALSE);
     itanalysis    -> SetMakeBranch(kFALSE);
     vtxanalysis   -> SetMakeBranch(kFALSE);
     calanalysis   -> SetMakeBranch(kFALSE);
     pfoanalysis   -> SetMakeBranch(kFALSE);
     jetanalysis   -> SetMakeBranch(kFALSE);

   } 
}
