#ifndef __J42LCIO__
#define __J42LCIO__

///////////////////////////////////////////////////////////////////////
//
//  J42LCIO
//
//  Jupiter output to LCIO Converer
//
//$Id:
//
///////////////////////////////////////////////////////////////////////

#include <TString.h>
#include <TObject.h>
#include "JSF2LCIO.h"

#if 0
#include "IMPL/SimTrackerHitImpl.h"
#include "IMPL/SimCalorimeterHitImpl.h"
#include "IMPL/LCCollectionVec.h"
#else
namespace IMPL {
  class LCCollectionVec;
  class LCEventImpl;
  class SimTrackerHitImpl;
  class SimCalorimeterHitImpl;
}
#endif
#include <map>

class JSFJ4Buf;
class JSFJ4;
class S4VExactHit;
class S4CALExactHit;

// *******************************************************
class J42LCIO : public JSF2LCIO
{
 protected:
  JSFJ4    *fJSFJ4;  //! Pointer to JSFJ4 object
  JSFJ4Buf *fBufp;   //! Pointer to JSFJ4 buffer

  IMPL::LCCollectionVec *fMCVec;  //! Pointer to CollectionVec for MC Particle
  Int_t fNPrimaries; // Number of primary particles
 public:
  J42LCIO(const char *name="J42LCIO", 
		  const char *title="J42LCIO");
  virtual ~J42LCIO();

  virtual Bool_t Initialize();
  virtual Bool_t BeginRun(Int_t runno=1);
  virtual Bool_t EndRun();
  virtual Bool_t Terminate();
  virtual Bool_t Process(Int_t ev=1);

  void SetMCParticles(IMPL::LCEventImpl *evt);
  void AssociateMCParticle(IMPL::SimTrackerHitImpl *hit, S4VExactHit *exact);
  void AssociateMCParticle(IMPL::SimCalorimeterHitImpl *hit, S4VExactHit *exact);
  void SetCALHits(IMPL::LCEventImpl *evt, std::multimap<Int_t,S4CALExactHit*> calhitmap, Bool_t isECAL=kTRUE);
  void SetBITHits(IMPL::LCEventImpl *evt);
  void SetFITHits(IMPL::LCEventImpl *evt);
  void SetVTXHits(IMPL::LCEventImpl *evt);
  void SetTPCHits(IMPL::LCEventImpl *evt);
  void SetMUDHits(IMPL::LCEventImpl *evt);
  
  void SetTPCMCParticles(JSFJ4Buf *bufp);
  void SetTPCPostMCParticles(JSFJ4Buf *bufp);
  void SetVTXMCParticles(JSFJ4Buf *bufp);
  void SetITMCParticles(JSFJ4Buf *bufp);

  std::multimap<Int_t,S4CALExactHit*> MakeCALHitsMap();

  inline JSFJ4Buf *GetBufferPointer(){ return fBufp;}
  inline JSFJ4    *GetJSFJ4(){ return fJSFJ4;}
  
  ClassDef(J42LCIO, 1)  // J42LCIO class

};

#endif










