//*************************************************************************
//* ----------------
//* S4BCALMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*
//* (Provides)
//*	S4BCALMCTruthBuf		: raw data buffer
//*	S4BCALMCTruth		: steering class
//* (Update Record)
//*	2006/02/01  A.Miyamoto  Prepared for BCAL
//*************************************************************************

#include "S4BCALMCTruth.h"
#include "S4BCALExactHit.h"
#include "S4BCALMCTruthBuf.h"
#include <iostream>
//#define __DEBUG__
using namespace std;

ClassImp(S4BCALMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4BCALMCTruth::S4BCALMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4BCALMCTruth is called..." << endl;
#endif
   fEventBuf = new S4BCALMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4BCALMCTruth::~S4BCALMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4BCALMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4BCALMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4BCALMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);

}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4BCALMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4BCAL`MCTruth::Process() is called..." << endl;  
#endif
   S4BCALMCTruthBuf *bufp = (S4BCALMCTruthBuf *)fEventBuf;

   if (fFile->IsWritable()) { // JSFJ4
      if (bufp->GetInputBufPtr()) {
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         ehitbufp->Clear();
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << " CALHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else { // Read ASCII data
	 if (!bufp->GetPreHitsPtr()) return kTRUE;
         bufp->GetExactHitsPtr()->Delete();
         bufp->GetPreHitsPtr()->Delete();
         bufp->ReadFile();
      } 
      bufp->UpdatePtrs();
#ifdef __DEBUG__
      cerr << "S4BCALMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#if 0
      DebugPrint();
#else
      Print();
#endif
#endif
      return kTRUE;
   }
   else {
      cerr << "S4BCALMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4BCALMCTruth::DebugPrint(const Char_t *)
{
   S4BCALMCTruthBuf *bufp = (S4BCALMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4BCALExactHit *ehitp;
   while ((ehitp = static_cast<S4BCALExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
