#ifndef __S4CALExactHit__
#define __S4CALExactHit__

///////////////////////////////////////////////////////////////////////
//
//  JSFJ4 CAL Hit data
//
//$Id: S4CALExactHit.h,v 1.10 2007/01/16 19:15:02 ono Exp $
//
///////////////////////////////////////////////////////////////////////

#include "JSFSteer.h"
#include "S4VExactHit.h"
#include <TVector3.h>

class J4CALHit;
class J4CLXHit;
class S4CALPreHit;

// *******************************************************
class S4CALExactHit : public S4VExactHit
{
 public:
  S4CALExactHit();
  S4CALExactHit(Int_t     hitID,       // hit ID
                Int_t     pcode,       // PDG code
          const TVector3  &pos,         // position [cm]
                Double_t  tof,         // TOF      [nsec]
                Double_t  edep,        // energy deposit [GeV]
                Int_t     preHitID,    // pre hit ID
                Int_t     cellID,      // cell ID
                Int_t     isBarrel,    // energy deposit [GeV]
                Int_t     isEM,        // energy deposit [GeV]
          const TVector3 &xcell,       // cell position [cm]
          const TVector3 &xcm,         // E-weighted pos [cm*GeV]
		Int_t     trackID,     // track ID
		Int_t     mtrackID);   // mother track ID
  S4CALExactHit(J4CALHit *ahit);
  S4CALExactHit(J4CLXHit *ahit);
  virtual ~S4CALExactHit();

  // virtual functions in S4VExactHit
  static void SetSortBy(Char_t *opt="");
  virtual Int_t Compare(const TObject*) const;
  virtual Bool_t IsSortable();

  inline Int_t        GetPreHitID(){ return fPreHitID; }
  inline Bool_t       IsBarrel(){ return fIsBarrel;  }
  inline Bool_t       IsEM(){ return fIsEM; }
  inline TVector3     GetCellPosition(){ return fCellPosition; }
  inline TVector3     GetCellXcm(){ return fCellXcm; }
  inline S4CALPreHit* GetPreHitPtr(){ return fPreHitPtr; }
  inline void         SetPreHitPtr(S4CALPreHit *p){fPreHitPtr = p; }
 
  virtual void Print(const Option_t *opt = "") const; //overrides TObject's

  //-------------------------------------------------------
  //* GetCellID was redefined to correct aug06 data overlap
  //inline Int_t        GetCellID(){ return fCellID; }
  Int_t  GetCellID();
  
 private:
  Int_t        fPreHitID;         // PreHitID
  Int_t        fCellID;           // CellID
  Bool_t       fIsBarrel;         // IsBarrel
  Bool_t       fIsEM;             // IsEM
  TVector3     fCellPosition;     // Cell position in cm
  TVector3     fCellXcm;          // Edep-weighted position
  S4CALPreHit *fPreHitPtr;        // pointer to pre hit 
  
  ClassDef(S4CALExactHit, 2)  // S4CALExactHit class

};

#endif
