//*************************************************************************
//* ----------------
//* S4FCALMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*
//* (Provides)
//*	S4FCALMCTruthBuf		: raw data buffer
//*	S4FCALMCTruth		: steering class
//* (Update Record)
//*	2006/02/01  A.Miyamoto  Prepared for FCAL
//*************************************************************************

#include "S4FCALMCTruth.h"
#include "S4FCALExactHit.h"
#include "S4FCALMCTruthBuf.h"
#include <iostream>
//#define __DEBUG__
using namespace std;

ClassImp(S4FCALMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4FCALMCTruth::S4FCALMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4FCALMCTruth is called..." << endl;
#endif
   fEventBuf = new S4FCALMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4FCALMCTruth::~S4FCALMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4FCALMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4FCALMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4FCALMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);

}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4FCALMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4FCAL`MCTruth::Process() is called..." << endl;  
#endif
   S4FCALMCTruthBuf *bufp = (S4FCALMCTruthBuf *)fEventBuf;

   if (fFile->IsWritable()) { // JSFJ4
      if (bufp->GetInputBufPtr()) {
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         ehitbufp->Clear();
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << " CALHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else { // Read ASCII data
	 if (!bufp->GetPreHitsPtr()) return kTRUE;
         bufp->GetExactHitsPtr()->Delete();
         bufp->GetPreHitsPtr()->Delete();
         bufp->ReadFile();
      } 
      bufp->UpdatePtrs();
#ifdef __DEBUG__
      cerr << "S4FCALMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#if 0
      DebugPrint();
#else
      Print();
#endif
#endif
      return kTRUE;
   }
   else {
      cerr << "S4FCALMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4FCALMCTruth::DebugPrint(const Char_t *)
{
   S4FCALMCTruthBuf *bufp = (S4FCALMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4FCALExactHit *ehitp;
   while ((ehitp = static_cast<S4FCALExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
