//*************************************************************************
//* ------------------
//* S4GENMCTruth Class
//* ------------------
//* (Description)
//*   	A class to store generator information.
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4GENMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2005/04/01  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "S4GENMCTruthBuf.h"
#include "S4VMCTruthBuf.h"
#include "S4PrimaryVertex.h"

using namespace std;

ClassImp(S4GENMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4GENMCTruthBuf::S4GENMCTruthBuf(const Char_t *name,
                                 const Char_t *title, 
                                 S4GENMCTruth *module,
                                 Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fOutputBufPtr(0),
                fDone(kFALSE),
                fPVcm(0.,0.,0.,0.)
{
}

S4GENMCTruthBuf::S4GENMCTruthBuf(S4GENMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fOutputBufPtr(0),
                fDone(kFALSE),
                fPVcm(0.,0.,0.,0.)
{
   cerr << "constructor for S4GENMCTruthBuf is called..." << endl;
}

//===========================================================
// Destructor -----------------------------------------------

S4GENMCTruthBuf::~S4GENMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4GENMCTruthBuf::ReadFile()
{
   return kTRUE;
}
 
//===========================================================
// Update  --------------------------------------------------

void S4GENMCTruthBuf::Update()
{
   if (fDone) return;
   fDone = kTRUE;

   Double_t ecm = 0.;
   TVector3 pcm;

   S4PrimaryVertex *pvp = static_cast<S4PrimaryVertex *>(GetInputBufPtr()->At(0));
   TIter next(pvp->GetParticles());
   S4PrimaryParticle *ppp;
   while ((ppp = static_cast<S4PrimaryParticle *>(next()))) {
      if (ppp->GetTrackID() >= 0 && !ppp->GetDaughters()->GetEntries()) {
         Double_t mass = ppp->GetMass();
         TVector3 p    = ppp->GetMomentum();
         pcm += p;
         ecm += TMath::Sqrt(p.Mag2() + mass*mass);
      }
   }
   fPVcm.SetXYZT(pcm.X(), pcm.Y(), pcm.Z(), ecm);
}
