//*************************************************************************
//* ----------------
//* S4MUDMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*
//* (Provides)
//*	S4MUDMCTruthBuf		: raw data buffer
//*	S4MUDMCTruth		: steering class
//* (Update Record)
//*	2004/11/24  S.Yamamoto	Original version.
//*************************************************************************

#include "S4MUDMCTruth.h"
#include "S4MUDExactHit.h"
#include "S4MUDMCTruthBuf.h"
#include <iostream>
//#define __DEBUG__
using namespace std;

ClassImp(S4MUDMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4MUDMCTruth::S4MUDMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4MUDMCTruth is called..." << endl;
#endif
   fEventBuf = new S4MUDMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4MUDMCTruth::~S4MUDMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4MUDMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4MUDMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4MUDMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);

}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4MUDMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4MUD`MCTruth::Process() is called..." << endl;  
#endif
   S4MUDMCTruthBuf *bufp = (S4MUDMCTruthBuf *)fEventBuf;

   if (fFile->IsWritable()) { // JSFJ4
      if (bufp->GetInputBufPtr()) {
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         ehitbufp->Clear();
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << " MUDHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else { // Read ASCII data
         if (!bufp->GetPreHitsPtr()) return kTRUE;
         bufp->GetExactHitsPtr()->Delete();
         bufp->GetPreHitsPtr()->Delete();
         bufp->ReadFile();
      } 
      bufp->UpdatePtrs();
#ifdef __DEBUG__
      cerr << "S4MUDMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#if 0
      DebugPrint();
#else
      Print();
#endif
#endif
      return kTRUE;
   }
   else {
      cerr << "S4MUDMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4MUDMCTruth::DebugPrint(const Char_t *)
{
   S4MUDMCTruthBuf *bufp = (S4MUDMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4MUDExactHit *ehitp;
   while ((ehitp = static_cast<S4MUDExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
