//*************************************************************************
//* --------------------
//* S4TPCExactHit
//* --------------------
//* (Description)
//*	TPC Hit class.
//*	Mother class : S4VExactHit
//*     
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************
#include <iostream>
#include <fstream>

#ifndef __HasJSFFromLCIO__
#include "J4TPCLayerHit.hh"
#endif

#include "S4TPCExactHit.h"
#include "JSFSteer.h"

ClassImp(S4TPCExactHit)

//=====================================================================
//* constructor -------------------------------------------------------

S4TPCExactHit::S4TPCExactHit()
             : fPadRowNumber(0), fPadRowRadius(0.), fPhi(0.),
               fSide(0), fZdrift(0.), fPrePosition(0.,0.,0.), fPostPosition(0.,0.,0.)
{
}    

#ifndef __HasJSFFromLCIO__
S4TPCExactHit::S4TPCExactHit(const J4TPCLayerHit *hitp)
             : S4VExactHit(0,                              // hit ID
                           hitp->GetTrackID(),             // track ID
                           hitp->GetMotherTrackID(),       // mother track ID
                           hitp->GetPDGEncoding(),         // particle ID
                           (Int_t)hitp->GetCharge(),       // charge
                           0.,                             // position
                           hitp->GetTof(),                 // TOF
                           (1 / GeV) * TVector3(hitp->GetMomentum().x(),
                                                hitp->GetMomentum().y(),
                                                hitp->GetMomentum().z()),
                           hitp->GetTotalEnergy()   / GeV, // total energy
                           hitp->GetEnergyDeposit() / GeV, // energy deposit
                           0,                              // address ptr
                           0),                             // event ID 
               fPadRowNumber(hitp->GetComponent()->GetMyID()), // pad raw No.
               fPrePosition((1/cm)*TVector3(hitp->GetPrePosition().x(),
                                            hitp->GetPrePosition().y(),
                                            hitp->GetPrePosition().z())),
               fPostPosition((1/cm)*TVector3(hitp->GetPostPosition().x(),
                                             hitp->GetPostPosition().y(),
                                             hitp->GetPostPosition().z()))
{
   JSFEnv   &env  = *gJSF->Env();
   static const Double_t halfz = env.GetValue("J4.TPC.HalfZ", 260.)
                               - env.GetValue("J4TPC.Endcap.HalfThickness", 1.) * 2
                               - env.GetValue("J4TPC.PadPlane.HalfThickness", 1.5) * 2;
   TVector3 pos = (fPrePosition.Pt() > fPostPosition.Pt()) ? fPostPosition
                                                           : fPrePosition;
   SetPosition(pos);
   Double_t z    = pos.Z();
   fZdrift       = z > 0 ? halfz - z : halfz + z;
   fPadRowRadius = pos.Pt();
   fPhi          = pos.Phi();
   fSide         = z > 0 ? 1 : -1;
}

#endif

S4TPCExactHit::S4TPCExactHit(Int_t     evtID,
                             Int_t     padrowNumber, 
                             Int_t     trackID,
                             Int_t     mothertrackID,
                             Int_t     hitID, 
                             Int_t     pcode, 
                             Int_t     charge,
                             const TVector3 &pre, 
                             const TVector3 &post, 
                             const TVector3 &momentum,
                             Double_t  totE, 
                             Double_t  edep, 
                             Double_t  tof)
             : S4VExactHit(hitID, trackID,  mothertrackID, pcode, charge,
                          (pre.Pt() > post.Pt()) ? post : pre,
                           tof, momentum, totE, edep,  0, evtID),
               fPadRowNumber(padrowNumber),
               fPrePosition(pre),
               fPostPosition(post)
{
   JSFEnv   &env  = *gJSF->Env();
   static const Double_t halfz = env.GetValue("J4.TPC.HalfZ", 260.)
                               - env.GetValue("J4TPC.Endcap.HalfThickness", 1.) * 2
                               - env.GetValue("J4TPC.PadPlane.HalfThickness", 1.5) * 2;
   TVector3 pos  = GetPosition();
   Double_t z    = pos.Z();
   fZdrift       = z > 0 ? halfz - z : halfz + z;
   fPadRowRadius = pos.Pt();
   fPhi          = pos.Phi();
   fSide         = z > 0 ? 1 : -1;
}


//=====================================================================
//* destructor --------------------------------------------------------

S4TPCExactHit::~S4TPCExactHit()
{
}

//=====================================================================
//* DebugPrint -----------------------------------------------------------

void S4TPCExactHit::DebugPrint(const Char_t *) 
{
   using namespace std;
   std::cerr << "------<S4TPCExactHit::DebugPrint>---------" << std::endl
        << " hit ID          = "  << GetHitID()         << std::endl
        << " track ID        = "  << GetTrackID()       << std::endl
        << " mother track ID = "  << GetMotherTrackID() << std::endl
        << " padrow number   = "  << GetPadRowNumber()  << std::endl
        << " pdg code        = "  << GetPDGCode()       << std::endl
        << " charge          = "  << GetCharge()        << std::endl
        << " position        = (" << GetPosition().X()  << ", "
                                  << GetPosition().Y()  << ", "
                                  << GetPosition().Z()  << ")" 
                                                        << std::endl
        << " momentum        = (" << GetMomentum().X()  << ", " 
                                  << GetMomentum().Y()  << ", " 
                                  << GetMomentum().Z()  << ")" 
                                                        << std::endl
        << " energy          = "  << GetTotalEnergy()   << std::endl
        << " edep            = "  << GetEnergyDeposit() << std::endl
        << " tof             = "  << GetTOF()           << std::endl
        << " radius          = "  << GetPosition().Pt() << std::endl;
}

