//*************************************************************************
//* --------------------
//* S4CALClusterMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes CALClusters.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include <iostream>
#include "S4CALCluster.h"
#include "S4CALClusterMakerBuf.h"
#include "S4CALHitMaker.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4HYBTrackMaker.h"
#include "S4HYBTrackMakerBuf.h"


//#define __USEPREHITID__
//#define __DEBUG__

ClassImp(S4CALClusterMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------

S4CALClusterMakerBuf::S4CALClusterMakerBuf(S4CALClusterMaker *module, 
                                           const Char_t      *name, 
                                           const Char_t      *title, 
                                           Int_t bufsize)
		    : JSFEventBuf(name, title, (JSFModule*)module),
		      fDone(kFALSE),
                      fInputDataBufPtr(0),
                      fOutputBufPtr(0),
                      fBufSize(bufsize)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALClusterMakerBuf::~S4CALClusterMakerBuf()
{
    fOutputBufPtr->Delete();
}

//===================================================================== 
//* GetInputBufPtr ----------------------------------------------------                                        

JSFEventBuf* S4CALClusterMakerBuf::GetInputDataBufPtr()
{
  return fInputDataBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ------------------------------------------------------

TObjArray* S4CALClusterMakerBuf::GetOutputBufPtr()
{
   return fOutputBufPtr;
}

//=====================================================================
//* GetEntries --------------------------------------------------------

Int_t S4CALClusterMakerBuf::GetEntries()
{
   if (!fDone) MakeClusters(); 
   return fOutputBufPtr->GetEntries();
}

//=====================================================================
//* SetInputDataBuf ---------------------------------------------------

void S4CALClusterMakerBuf::SetInputDataBufPtr(JSFEventBuf *buf)
{
   fDone = kFALSE;
   fInputDataBufPtr = buf;
   if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
   if (fOutputBufPtr->GetEntries() > 0) fOutputBufPtr->Delete();
}
//=====================================================================
//* DebugPrint --------------------------------------------------------
 
void S4CALClusterMakerBuf::DebugPrint(const Char_t *)
{
   if (fOutputBufPtr && fOutputBufPtr->GetEntries()) {
      TIter next(fOutputBufPtr);
      S4CALCluster *cp = 0;
      while ((cp = static_cast<S4CALCluster *>(next()))) {
         cp->DebugPrint();
      }
   }
}

//=====================================================================
//* MakeClusters ------------------------------------------------------

void S4CALClusterMakerBuf::MakeClusters()
{
   if (fDone) return; // check if clusters already made 

   fDone = kTRUE;
   fOutputBufPtr->SetOwner();        // buffer owns clusters 
   //--
   // First check if there are any hits to cluster
   //--
   TObjArray &calHits = * static_cast<S4CALHitBuf *>(GetInputDataBufPtr())
                          ->GetOutputBufPtr();
   if (!&calHits || calHits.GetEntries() == 0) return; 
   //--
   // Sort hits by parent ID
   //--
#ifdef __USEPREHITID__
   S4CALHit::SetSortBy(S4CALHit::kPreHitID);  // sort hits by prehitID
#else
   S4CALHit::SetSortBy(S4CALHit::kPostHitID); // sort hits by posthitID
#endif
   calHits.UnSort();
   calHits.Sort();
   //--
   // Check if any corresponding hybrid track there
   //--
   TIter nexthit(&calHits); // prepare hit iterator
#ifdef  __USEPREHITID__
   S4CALHit      *hitp  = static_cast<S4CALHit *>(nexthit());
   S4CALExactHit *ehitp = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
   S4CALPreHit   *phitp = ehitp->GetPreHitPtr();
   Int_t        j4trkid = phitp->GetTrackID();
#else
   //--
   // Separate out orphan hits
   //  Orphan hits have postHit ID = -1, making an orphan group in
   //  the starting part of calHits, when sorted by postHit ID
   //--
   TObjArray      strayHits;  // array to store orphan hits
   S4CALHit      *hitp  = 0;
   S4CALExactHit *ehitp = 0;
   S4CALPreHit   *phitp = 0;
   S4TPCPostHit  *poshp = 0;
   while ((hitp = static_cast<S4CALHit *>(nexthit()))) { // loop over all hits
      ehitp = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
      phitp = ehitp->GetPreHitPtr();
      poshp = phitp->GetPostHitPtr();
      if (poshp) break;               // end of orphan hits
      else       strayHits.Add(hitp); // store this to orphan array
   }
   static const Int_t kAllOrphans = -9999;
                Int_t j4trkid = poshp ? poshp->GetTrackID() : kAllOrphans;
#endif
   //--
   // First process CAL hits with parent hits
   //--
   // Create 1st cluster
   S4HYBTrackMakerBuf *htmbp = static_cast<S4HYBTrackMakerBuf *>(
                                  static_cast<S4HYBTrackMaker *>(
                                     gJSF->FindModule("S4HYBTrackMaker"))
                               ->EventBuf());
   S4HYBTrack   *trkp   = htmbp->GetHYBTrackPtr(j4trkid);
   S4CALCluster *curcp  = new S4CALCluster(trkp); // new 1st cluster
   if (trkp) trkp->Lock();                        // lock the track if linked
   if (hitp) curcp->Append(hitp);                 // append this to the 1st cluster

   // Then create the rest
   while ((hitp = static_cast<S4CALHit *>(nexthit()))) { // loop over remaining hits
      if (!curcp->Append(hitp)) {   // try append hit to current cluster
         fOutputBufPtr->Add(curcp); // failed, end of current cluster, add current to buffer 
         // check if there is any corresponding hybrid track
	 ehitp   = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
	 phitp   = ehitp->GetPreHitPtr();
#ifdef  __USEPREHITID__
	 j4trkid = phitp->GetTrackID();
#else
         poshp   = phitp->GetPostHitPtr();
         j4trkid = poshp ? poshp->GetTrackID() : phitp->GetTrackID();
#endif
         trkp    = htmbp->GetHYBTrackPtr(j4trkid); 

	 curcp = new S4CALCluster(trkp); // start new cluster
         if (trkp) trkp->Lock();
	 curcp->Append(hitp); // append this hit to new cluster 
      }
   }
   if (curcp->GetEntries()) fOutputBufPtr->Add(curcp); // add last cluster to buffer
   else                     delete curcp;
   //--
   // Then process stray (orphan) hits, if any
   //--
   if (strayHits.GetEntries()) {
      S4CALHit::SetSortBy(S4CALHit::kPreHitID);  // sort hits by prehitID
      strayHits.UnSort();
      strayHits.Sort();

      TIter nextshit(&strayHits);
      hitp    = static_cast<S4CALHit *>(nextshit());
      ehitp   = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
      phitp   = ehitp->GetPreHitPtr();
      j4trkid = phitp->GetTrackID();
      trkp    = htmbp->GetHYBTrackPtr(j4trkid); 

      // Create 1st cluster
      curcp   = new S4CALCluster(trkp); // new 1st cluster
      if (trkp) trkp->Lock();           // lock the track if linked
      curcp->Append(hitp);              // append this hit to the 1st cluster

      // Then create the rest
      while ((hitp = static_cast<S4CALHit *>(nextshit()))) { // loop over remaining hits
         if (!curcp->Append(hitp)) {   // try append hit to current cluster
            fOutputBufPtr->Add(curcp); // failed, end of current cluster, add current to buffer 

            // check if there is any corresponding hybrid track
            ehitp   = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
            phitp   = ehitp->GetPreHitPtr();
	    j4trkid = phitp->GetTrackID();
            trkp    = htmbp->GetHYBTrackPtr(j4trkid); 

	    curcp = new S4CALCluster(trkp); // start new cluster
            if (trkp) trkp->Lock();
            curcp->Append(hitp); // append this hit to new cluster 
         }
      }
      if (curcp->GetEntries()) fOutputBufPtr->Add(curcp); // add last cluster to buffer
      else                     delete curcp;
   }

#ifdef __DEBUG__
   DebugPrint();
#endif
   //--
   // That's it, folks!
   //--
}
