#ifndef __S4CALHIT__
#define __S4CALHIT__
//*****************************************************************************
//* ==============
//* S4CALHit
//* ==============
//*   METIS (smeared) CAL hits.
//*   Mother class: S4VHit
//*
//* (Update Record)
//*   2004.07.30  A.Sanchez  Primitive version.
//*****************************************************************************
#include "S4VHit.h"
#include "TVector3.h"
#include "TAttDrawable.h"
//#include "S4CALTypeDef.h"

class S4CALExactHit;

class S4CALHit : public S4VHit, public TAttDrawable
{

public:

  S4CALHit();
  S4CALHit(Int_t caltype, Double_t edep, Double_t tof,
	   TVector3 pos, Int_t clustype = 0);
  S4CALHit(S4CALExactHit *h);
  virtual ~S4CALHit();

  void DebugPrint(const Char_t *opt = "");

  virtual Bool_t IsSortable() const { return kTRUE; }
  virtual Int_t  Compare(const TObject* obj) const;

public:

  // Data Getters
  inline Int_t      GetClusterType() const { return fClusterType; }
  inline Int_t      GetCALType()     const { return fCALType;     }
  inline Int_t      GetCellID() const { return fCellID; }
  inline void       SetCellID(Int_t ce) { fCellID = ce; }
  inline TVector3   GetCellPosition() const { return fCellPosition; }
  virtual Bool_t     IsEM() const { return fIsEM; }

  // Data Setters
  void SetClusterType(Int_t t)  { fClusterType = t; }

  // Draw Method
  using TAttDrawable::Draw;
  virtual void Draw(Int_t color, const Char_t *opt = "");

private:

  // hit type
  Int_t    fCALType;	   // calorimeter type
  Int_t    fClusterType;   // cluster type
  Int_t    fCellID;        // CellID 
  TVector3 fCellPosition;  // Cell position in cm    
  Bool_t   fIsEM;          // IsEM

  ClassDef(S4CALHit, 1) // a CAL (smeared) hit
};

#endif
