//*************************************************************************
//* ---------------------
//* S4CALClusterUtils Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALClusterUtils.h"
#include "S4CALAddress.h"
#include "S4CALTypeDef.h"
#include "JSFHelicalTrack.h"
#include "JSFSteer.h"
#include "JSFGenerator.h"
#include "JSFGeneratorParticle.h"
#include "S4CALSmallClusterMakerBuf.h"
#include "TVirtualFitter.h"

#include "TNtuple.h"
#include "TGraphErrors.h"
#include "TF1.h"

#include <iostream>

#define __LRATIO__

//#define __LOGLIKE__

Double_t S4CALClusterUtils::fMissingEnergy   = 0.;
Double_t S4CALClusterUtils::fNeutrinoEnergy  = 0.;
Bool_t   S4CALClusterUtils::fDoneCalcMissing = kFALSE;
Int_t    S4CALClusterUtils::fNEvt            = -1;

ClassImp(S4CALClusterUtils)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALClusterUtils::S4CALClusterUtils()
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALClusterUtils::~S4CALClusterUtils()
{
}

//==========================================================================
//* MakeSampleHistogram ----------------------------------------------------

std::map<Double_t,Double_t> S4CALClusterUtils::MakeSampleHistogram(TTree* sample, Char_t* param, const Char_t* selection, const Int_t nbins, const Double_t max)
{

  std::map<Double_t,Double_t> histmap;

  Double_t xmax = max;
  Double_t xmin = 0.;
  Double_t norm = 1.;
  Double_t dx   = 0.;

  //  sample->Project("HISTINMAKESAMPLEHISTOGRAM",param,selection);
  //  TH1D *hist = (TH1D*) gDirectory->FindObject("HISTINMAKESAMPLEHISTOGRAM");
  //  xmax = hist->GetXaxis()->GetXmax();
  //  xmin = hist->GetXaxis()->GetXmin();
  //  dx   = (xmax-xmin)/nbins;

  TH1D *hist2 = new TH1D("HISTINMAKESAMPLEHISTOGRAM2","",nbins,xmin,xmax);
  //  TH1D *histb = new TH1D("HISTINMAKESAMPLEHISTOGRAMB","",nbins,xmin,xmax);
  sample->Project("HISTINMAKESAMPLEHISTOGRAM2",param,selection);
  //  sample->Project("HISTINMAKESAMPLEHISTOGRAMB",param,"edep-egam");
  if(hist2->Integral(1,nbins) < 1.e-30 ){
    std::cerr << "!!!! check selection criteria !!!!" << std::endl;
    //    gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAM");
    gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAM2");
    //    gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAMB");
    return histmap;
  }
  dx   = (xmax-xmin)/nbins;  
  norm = hist2->Integral(1,nbins+1); // count overflow bin
  hist2->Scale(1./norm);
  //  hist2->Divide(histb);
  for(Int_t n = 0; n < nbins+2; n++){
    Double_t x    = xmin + (n - 0.5)*dx;
    //    Int_t    bin  = hist2->GetXaxis()->FindBin(x);
    Int_t    bin  = hist2->FindBin(x);
    Double_t prob = hist2->GetBinContent(bin);
    histmap[xmin+n*dx] = prob;
  }
  //  gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAM");
  gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAM2");
  //  gDirectory->Delete("HISTINMAKESAMPLEHISTOGRAMB");

  return histmap;

}

//==========================================================================
//* CalcProbability --------------------------------------------------------

Double_t S4CALClusterUtils::CalcProbability(const Double_t value, std::map<Double_t,Double_t> histmap)
{

  Double_t prob = 0.;

  if(histmap.size() < 1) return prob;
#if 1

  std::map<Double_t,Double_t>::reverse_iterator ritr;
  if(value > histmap.rbegin()->first){
    prob = histmap.rbegin()->second; // overflow
    return prob;
  }

  for(ritr = histmap.rbegin(); ritr != histmap.rend(); ritr++){
    if (value <= (ritr->first)) prob = ritr->second;
    else                        break;
  }

#else

  std::map<Double_t,Double_t>::iterator itr;

  itr = histmap.upper_bound(value);
  if(itr == histmap.end())   prob = 0.;
  else                       prob = itr->second;

#endif

  return prob;

}

#ifndef __LOGLIKE__
//==========================================================================
//* CalcLikelihood ---------------------------------------------------------

Double_t S4CALClusterUtils::CalcLikelihood(const Int_t nparams, const Int_t nbgkind, const TMatrixD sprob, const TMatrixD bgprob)
{

  Double_t likelihood = 0.;
  Double_t nrows = sprob.GetNrows();
  Double_t ncols = sprob.GetNcols();
  Double_t nrowb = bgprob.GetNrows();
  Double_t ncolb = bgprob.GetNcols();


  if(nrows != nparams || ncols != 1 || nrowb != nparams || ncolb != nbgkind){
    std::cerr << "set nparams/nbgkind correctly!!" << std::endl;
    return likelihood;
  }
#ifdef __LRATIO__
  Double_t signal = 1.;
  Double_t bkg[nbgkind];
  for(Int_t m = 0; m < nbgkind; m++)  bkg[m] = 1.;

  for(Int_t n = 0; n < nparams; n++){
    signal *= sprob(n,0);
    for(Int_t m = 0; m < nbgkind; m++){
      bkg[m] *= bgprob(n,m);
    }
  }
  
  Double_t sum = signal;
  for(Int_t m = 0; m < nbgkind; m++)   sum += bkg[m];

  if(sum > 1.e-15)  likelihood = signal/sum;
  else              likelihood = 0.;
#else
  likelihood = 1.;
  for(Int_t n = 0; n < nparams; n++)  likelihood *= sprob(n,0);
#endif
  return likelihood;

}
#else
//==========================================================================
//* CalcLikelihood ---------------------------------------------------------

Double_t S4CALClusterUtils::CalcLikelihood(const Int_t nparams, const Int_t nbgkind, const TMatrixD sprob, const TMatrixD bgprob)
{

  Double_t likelihood = 0.;
  Double_t nrows = sprob.GetNrows();
  Double_t ncols = sprob.GetNcols();
  Double_t nrowb = bgprob.GetNrows();
  Double_t ncolb = bgprob.GetNcols();


  if(nrows != nparams || ncols != 1 || nrowb != nparams || ncolb != nbgkind){
    std::cerr << "set nparams/nbgkind correctly!!" << std::endl;
    return likelihood;
  }
#ifdef __LRATIO__
  Double_t signal = 1.;
  Double_t bkg[nbgkind];
  for(Int_t m = 0; m < nbgkind; m++)  bkg[m] = 1.;

  for(Int_t n = 0; n < nparams; n++){
    signal *= sprob(n,0);
    for(Int_t m = 0; m < nbgkind; m++){
      bkg[m] *= bgprob(n,m);
    }
  }
  
  Double_t sum = signal;
  for(Int_t m = 0; m < nbgkind; m++)   sum += bkg[m];

  likelihood = TMath::Log(signal) - TMath::Log(bkg[0]);

#else
  likelihood = 1.;
  for(Int_t n = 0; n < nparams; n++)  likelihood *= sprob(n,0);
#endif
  return likelihood;

}
#endif

//==========================================================================
//* EHMatcher --------------------------------------------------------------

void S4CALClusterUtils::EHMatcher(std::map<Int_t, S4CALSmallCluster*> &sclmap, const Double_t matchcut, const Int_t minhits)
{

  std::map<Int_t, S4CALSmallCluster*>::iterator emitr;
  std::map<Int_t, S4CALSmallCluster*>::iterator hditr;

  for(emitr = sclmap.begin(); emitr != sclmap.end(); emitr++){
    
    TVector3 empos(0.,0.,0.);
    Double_t distance  = 0.;

    // If small cluster is already locked, it will not be connected.
    if ( !(emitr->second->IsEM()) || (emitr->second->IsLocked()) ) continue;
    if(emitr->second->IsMIP() || emitr->second->GetNHits() < minhits) continue;

    empos = emitr->second->GetMaxLayerPos();

    // loop over small clusters within HCAL
    for(hditr = sclmap.begin(); hditr != sclmap.end(); hditr++){

      if(hditr->second->IsMIP() || hditr->second->GetNHits() < minhits) continue;
      if( (hditr->second->IsEM()) || hditr->second->IsLocked() ) continue;

      TVector3 hdpos(0.,0.,0.);

      hdpos = hditr->second->GetMinLayerPos();
      distance = (empos - hdpos).Mag();
      if ( distance < matchcut ) {
	std::vector<Double_t> vec;
	// register in HD small cluster
	vec.clear();
	vec.push_back(hditr->second->GetClusterID()); // matched pair ID
	vec.push_back(distance);
	(emitr->second->GetPFOMap()).insert(std::pair<MatchedType_t, std::vector<Double_t> >(kgEHMatcher, vec));
	// register in EM small cluster
	vec.clear();
	vec.push_back(emitr->second->GetClusterID()); // matched pair ID
	vec.push_back(distance);
	(hditr->second->GetPFOMap()).insert(std::pair<MatchedType_t, std::vector<Double_t> >(kgEHMatcher, vec));
      }

    } // end of hditr

  } // end of emitr

}

//==========================================================================
//* dXdTCut ----------------------------------------------------------------

Bool_t S4CALClusterUtils::dXdTCut(S4CALSmallCluster *mother, S4CALSmallCluster *child, const Double_t cutx, const Double_t cutt)
{

  TVector3 mpos = mother -> GetPosition();
  TVector3 cpos = child  -> GetPosition();
  TVector3 subp = mpos - cpos;
  //subp *= ( (child->GetEnergy())/(mother->GetEnergy()+child->GetEnergy()));

  Double_t subt = mother->GetTOF() - child->GetTOF();

  if( (subp.Mag() < cutx) && (TMath::Abs(subt) < cutt) ) return kTRUE;
  else  return kFALSE;


}

//==========================================================================
//* IsWithinTube -----------------------------------------------------------

Bool_t S4CALClusterUtils::IsWithinTube(const TVector3 pos, const TVector3 centerpos, const TVector3 axis, const Double_t tubeR, const Double_t tubeH)
{

  if( ((pos - centerpos).Perp(axis) < tubeR) && ((pos - centerpos).Mag() < tubeH) ){
    return kTRUE;
  }else{
    return kFALSE;
  }

}

//==========================================================================
//* FitByLinear ------------------------------------------------------------

std::vector<Double_t> S4CALClusterUtils::FitByLinear(std::map<Int_t,S4CALSmallCluster*> &sclmap, S4CALSmallCluster* scls, Double_t tubeR, Double_t tubeH, Char_t *opt)
{

  TVirtualFitter::SetMaxIterations(1000);

  //  static const Int_t emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Int_t hdnlayers = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
  static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
  static const Int_t hdnlayers = S4CALAddress::GetHDNLayers();
  static const Int_t nlayers   = emnlayers + hdnlayers;
  //  static const Double_t emtotalradlen = 27.;
  //  static const Double_t hdtotalradlen = 184.7;
  static const Double_t emtotalradlen = 26.;
  static const Double_t hdtotalradlen = 165.;

  std::vector<Double_t> paramv;

  if( (opt[0] == 'E') && !(scls->IsEM()) )     return paramv; // check EM only
  else if( (opt[0] == 'H') && (scls->IsEM()) ) return paramv; // check HD only

  TVector3 mpos    = scls->GetPosition();
  TVector3 mthrust = scls->GetThrustVector();

  if(mthrust.Mag() < 1.e-10) mthrust = mpos*(1./(mpos.Mag()));
 
 //-------------- Initialize ----------------------
  Double_t radlength[nlayers];
  for(Int_t i = 0; i < nlayers; i++){
    if(i < emnlayers) radlength[i] = emtotalradlen*(i+1)/emnlayers;
    else radlength[i] = emtotalradlen + hdtotalradlen*(i-emnlayers+1)/hdnlayers;
  }

  Double_t energy[nlayers];  // e-dep in each layer
  Int_t layerhits[nlayers];  // number of hits in each layer
  Double_t   yerr[nlayers];
  Double_t   xerr[nlayers];
  
  for(Int_t loop = 0; loop < nlayers; loop++){
    energy[loop]    = 0.;
    layerhits[loop] = 0;
    yerr[loop]      = 0;
    xerr[loop]      = 0;
  }
  //------------------------------------------------

  //-------- sum up energy deposit -----------------
  TVector3 childpos = TVector3(0., 0., 0.);

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;

  Int_t    maxlayerID = -1;
  Int_t    minlayerID = 10000;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if( (opt[0] == 'E') && !(scls->IsEM()) )     continue;
    else if( (opt[0] == 'H') && (scls->IsEM()) ) continue;

    if( IsWithinTube(itr->second->GetPosition(),mpos,mthrust,tubeR,tubeH) ){

      childpos   = itr->second->GetPosition();

      std::vector<S4CALHit*> calhits = itr->second->GetCALHitsVector();
      for(Int_t hit = 0; hit < (Int_t) calhits.size(); hit++ ){

	//! layer ID is counted from 0
	Int_t layerID = S4CALAddress::GetLayerID(calhits[hit]->GetCellID(),calhits[hit]->IsEM());

	if(!itr->second->IsEM()) layerID += emnlayers;

	if(maxlayerID < layerID) maxlayerID = layerID;
	if(minlayerID > layerID) minlayerID = layerID;

	energy[layerID] += calhits[hit]->GetEnergyDeposit();
	layerhits[layerID]++;
	
	if(layerID > 0){
	  energy[layerID-1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID-1]++;
	}else{  // layer ID = 0
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	if(layerID < nlayers-1){ // layer ID != maximum layer ID
	  energy[layerID+1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID+1]++;
	}else{
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	
      }
      
    } // end of IsWithinTube(...) loop

  } // end of small cluster map loop

  //------------------------------------------------

  if(maxlayerID - minlayerID < 1) return paramv;
    
  //--- temporary error!! ------------------
  for(Int_t k = 0; k < nlayers; k++){
    if(layerhits[k] > 0){
      energy[k] *= 1.e+6; // GeV -> keV
      yerr[k] = energy[k]/(TMath::Sqrt((Double_t)layerhits[k]));
    }else{
      yerr[k] = 1.e+6;
    }
    xerr[k] = 0.;
  }
  //----------------------------------------

  //------- Fit -----------
  TGraphErrors graph = TGraphErrors(nlayers, radlength, energy, xerr, yerr); 

  Double_t xmin;
  Double_t xmax;
  (minlayerID < emnlayers) ?
    xmin = (minlayerID+1)*emtotalradlen/emnlayers:
    xmin = emtotalradlen + (minlayerID-emnlayers+1)*hdtotalradlen/hdnlayers;
  (maxlayerID < emnlayers) ?
    xmax = (maxlayerID+1)*emtotalradlen/emnlayers:
    xmax = emtotalradlen + (maxlayerID-emnlayers+1)*hdtotalradlen/hdnlayers;

  TF1 func = TF1("FUNCINFITBYLINEAR","[0]*x + [1]", xmin, xmax);
  func.SetParameters(0., 0.5);
  graph.Fit(&func, "QN", "", xmin, xmax);
  paramv.push_back(func.GetParameter(0));
  paramv.push_back(func.GetParError(0));
  paramv.push_back(func.GetParameter(1));
  paramv.push_back(func.GetParError(1));
  paramv.push_back(func.GetChisquare()/func.GetNDF());
  //-----------------------

  return paramv;

}

//==========================================================================
//* FitByQuadratic ---------------------------------------------------------

std::vector<Double_t> S4CALClusterUtils::FitByQuadratic(std::map<Int_t,S4CALSmallCluster*> &sclmap, S4CALSmallCluster* scls, Double_t tubeR, Double_t tubeH, Char_t *opt)
{

  TVirtualFitter::SetMaxIterations(5000);

  //  static const Int_t emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Int_t hdnlayers = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
  static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
  static const Int_t hdnlayers = S4CALAddress::GetHDNLayers();
  static const Int_t nlayers   = emnlayers + hdnlayers;
  //  static const Double_t emtotalradlen = 27.;   // total radiation length
  //  static const Double_t hdtotalradlen = 184.7;
  static const Double_t emtotalradlen = 26.;   // total radiation length
  static const Double_t hdtotalradlen = 165.;

  std::vector<Double_t> paramv;

  if( (opt[0] == 'E') && !(scls->IsEM()) )     return paramv; // check EM only
  else if( (opt[0] == 'H') && (scls->IsEM()) ) return paramv; // check HD only

  TVector3 mpos    = scls->GetPosition();
  TVector3 mthrust = scls->GetThrustVector();

  if(mthrust.Mag() < 1.e-10) mthrust = mpos*(1./(mpos.Mag()));
 
 //-------------- Initialize ----------------------
  Double_t radlength[nlayers];
  for(Int_t i = 0; i < nlayers; i++){
    if(i < emnlayers) radlength[i] = emtotalradlen*(i+1)/emnlayers;
    else radlength[i] = emtotalradlen + hdtotalradlen*(i-emnlayers+1)/hdnlayers;
  }

  Double_t energy[nlayers];  // e-dep in each layer
  Int_t layerhits[nlayers];  // number of hits in each layer
  Double_t   yerr[nlayers];
  Double_t   xerr[nlayers];
  
  for(Int_t loop = 0; loop < nlayers; loop++){
    energy[loop]    = 0.;
    layerhits[loop] = 0;
    yerr[loop]      = 0;
    xerr[loop]      = 0;
  }
  //------------------------------------------------

  //-------- sum up energy deposit -----------------
  TVector3 childpos = TVector3(0., 0., 0.);

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;

  Int_t    maxlayerID = -1;
  Int_t    minlayerID = 10000;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if( (opt[0] == 'E') && !(scls->IsEM()) )     continue;
    else if( (opt[0] == 'H') && (scls->IsEM()) ) continue;

    if( IsWithinTube(itr->second->GetPosition(),mpos,mthrust,tubeR,tubeH) ){

      childpos   = itr->second->GetPosition();

      std::vector<S4CALHit*> calhits = itr->second->GetCALHitsVector();
      for(Int_t hit = 0; hit < (Int_t) calhits.size(); hit++ ){

	//! layer ID is counted from 0
	Int_t layerID = S4CALAddress::GetLayerID(calhits[hit]->GetCellID(),calhits[hit]->IsEM());

	if(!itr->second->IsEM()) layerID += emnlayers;

	if(maxlayerID < layerID) maxlayerID = layerID;
	if(minlayerID > layerID) minlayerID = layerID;

	energy[layerID] += calhits[hit]->GetEnergyDeposit();
	layerhits[layerID]++;
	
	if(layerID > 0){
	  energy[layerID-1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID-1]++;
	}else{  // layer ID = 0
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	if(layerID < nlayers-1){ // layer ID != maximum layer ID
	  energy[layerID+1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID+1]++;
	}else{
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	
      }
      
    } // end of IsWithinTube(...) loop

  } // end of small cluster map loop

  //------------------------------------------------

  if(maxlayerID - minlayerID < 1) return paramv;
    
  //--- temporary error!! ------------------
  for(Int_t k = 0; k < nlayers; k++){
    if(layerhits[k] > 0){
      energy[k] *= 1.e+6;  // GeV -> keV
      yerr[k] = energy[k]/(TMath::Sqrt((Double_t)layerhits[k]));
    }else{
      yerr[k] = 1.e+6;
    }
    xerr[k] = 0.;
  }
  //----------------------------------------

  //------- Fit -----------
  TGraphErrors graph = TGraphErrors(nlayers, radlength, energy, xerr, yerr); 

  Double_t xmin;
  Double_t xmax;
  (minlayerID < emnlayers) ?
    xmin = (minlayerID+1)*emtotalradlen/emnlayers:
    xmin = emtotalradlen + (minlayerID-emnlayers+1)*hdtotalradlen/hdnlayers;
  (maxlayerID < emnlayers) ?
    xmax = (maxlayerID+1)*emtotalradlen/emnlayers:
    xmax = emtotalradlen + (maxlayerID-emnlayers+1)*hdtotalradlen/hdnlayers;

  TF1 func = TF1("FUNCINFITBYQUADRATIC","[0]*(x-[1])*(x-[1])+[2]", xmin, xmax);
  func.SetParameters(0., 0., 0.5);
  graph.Fit(&func, "QN", "", xmin, xmax);
  paramv.push_back(func.GetParameter(0));
  paramv.push_back(func.GetParError(0));
  paramv.push_back(func.GetParameter(1));
  paramv.push_back(func.GetParError(1));
  paramv.push_back(func.GetParameter(2));
  paramv.push_back(func.GetParError(2));
  paramv.push_back(func.GetChisquare()/func.GetNDF());
  //-----------------------

  return paramv;

}

//==========================================================================
//* CalcIsolation ----------------------------------------------------------

Double_t S4CALClusterUtils::CalcIsolation(std::map<Int_t,S4CALSmallCluster*> &sclmap, S4CALSmallCluster* scl, const Double_t tubeRin, const Double_t tubeRout, const Double_t tubeH)
{

  Double_t ratio = -1.;
  Double_t Ein   = 0.;
  Double_t Eout  = 0.;
  TVector3 motherpos = scl->GetPosition();
  TVector3 thrustv   = scl->GetThrustVector();

  //if(thrustv.Mag() < 1.e-10) return ratio;
  if(thrustv.Mag() < 1.e-10){
    thrustv = scl->GetPosition()*(1./(scl->GetPosition()).Mag());
  }

  std::map<Int_t,S4CALSmallCluster*>::iterator itr;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;

    if(IsWithinTube(itr->second->GetPosition(),motherpos,thrustv,tubeRin,tubeH))
      Ein  += itr->second->GetEnergy();
    else if(IsWithinTube(itr->second->GetPosition(), motherpos, thrustv, tubeRout, tubeH))
      Eout += itr->second->GetEnergy();

  }

  ratio = Eout/Ein;

  return ratio;

}

//==========================================================================
//* DirectionIsMatched -----------------------------------------------------
Bool_t S4CALClusterUtils::DirectionIsMatched(TVector3 v1, TVector3 v2, Double_t cutangle)
{
  if( v1.Angle(v2) < cutangle ) return kTRUE;
  else                          return kFALSE;
}


////////////////////////////////////////////////////////////////////////////
// Cheating!!                                                             //
//                                                                        //

//==========================================================================
//* CalcMissingEnergy ------------------------------------------------------

void S4CALClusterUtils::CalcMissingEnergy()
{

  //  static const Double_t eccalinR = gJSF->Env()->GetValue("J4CAL.Endcap.InnerRadius",39.9);
  //  static const Double_t eccalz   = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
  static const Double_t eccalinR = S4CALAddress::GetEndcapInnerR();
  static const Double_t eccalz   = S4CALAddress::GetBarrelHalfZ();
  static const Double_t bfield   = gJSF->Env()->GetValue("J4SOL.BField",3.);

  if(fNEvt != gJSF->GetEventNumber()){
    fNEvt = gJSF->GetEventNumber();
    fDoneCalcMissing = kFALSE;
  }

  if(fDoneCalcMissing) return;

  fDoneCalcMissing = kTRUE;

  fNeutrinoEnergy  = 0.;
  fMissingEnergy   = 0.;

  // -------------------------------------------------------
  // Loop over GeneratorParticleList 
  // -------------------------------------------------------
  JSFGenerator *gen = (JSFGenerator*)gJSF -> FindModule("JSFGenerator");
  if(!gen) return;
  JSFGeneratorBuf *genb=(JSFGeneratorBuf*)gen -> EventBuf();
  TClonesArray *gps = genb -> GetParticles();

  for(Int_t i = 0; i < genb->GetNparticles(); i++) {

    JSFGeneratorParticle *gp = (JSFGeneratorParticle*) gps->UncheckedAt(i);

    if( gp -> GetNDaughter() > 0 ) continue;  // Select stable particles

    Int_t pid = TMath::Abs(gp -> GetID());
    Double_t energy = gp->GetE();

    if( (pid == 12) || (pid == 14) || (pid == 16) ){
      fNeutrinoEnergy  += energy;
      fMissingEnergy   += energy;
      continue;
    }

    TVector3 pv = TVector3(gp->GetPx(), gp->GetPy(), gp->GetPz());
    Double_t charge   = gp->GetCharge();
    Double_t pt       = TMath::Sqrt((pv.X())*(pv.X()) + (pv.Y())*(pv.Y()));

    // temporary!
    if(TMath::Abs(charge) > 0.){
      if( ((2.e+3)*pt/(3*bfield) < eccalinR) || (eccalinR/eccalz > pt/TMath::Abs(pv.Z())) )
	fMissingEnergy += energy;
    }else{
      if( eccalinR/eccalz > pt/TMath::Abs(pv.Z()) )
	fMissingEnergy += energy;
    }

  }

}

//==========================================================================
//* GetMissingEnergy -------------------------------------------------------

Double_t S4CALClusterUtils::GetMissingEnergy()
{
  CalcMissingEnergy();
  return fMissingEnergy;
}

//==========================================================================
//* GetNeutrinoEnergy ------------------------------------------------------

Double_t S4CALClusterUtils::GetNeutrinoEnergy()
{
  CalcMissingEnergy();
  return fNeutrinoEnergy;
}

