#ifndef __S4CALTRACKMATCHER__
#define __S4CALTRACKMATCHER__

//*************************************************************************   
//* ---------------------                      
//* S4CALTrackMatcher Class
//* ---------------------   
//* (Description) 
//*     represent calorimeter cluster
//* (Requires)
//*     S4CALHit
//* (Provides)  
//*     S4CALRealCluster            : made object class
//* (Update Record)
//*     2004/09/03  S.Yamamoto  Original version.
//*     2004/11/15  S.Yamamoto  Add IsOK
//*************************************************************************

#include "TObjArray.h"
#include "TVector3.h"
#include "S4CALSmallCluster.h"
#include "S4HYBTrack.h"

#include <utility>
#include <map>
#include <vector>

class S4CALTrackMatcher {

 public:
  S4CALTrackMatcher();
  virtual ~S4CALTrackMatcher();

 public:

 //---------------------------------------------------------
 // thrust based track matching
 //---------------------------------------------------------
 virtual void TrackThrustMatching(S4CALSmallCluster* scls,
				  S4HYBTrack *trk,
				  Double_t cutperp = 10.); // (cm)

 //---------------------------------------------------------
 // tube based track matching
 //---------------------------------------------------------
  virtual void TrackTubeMatching(S4CALSmallCluster* scls,
				 S4HYBTrack *trk,
				 Double_t radius = 10.,
				 Double_t slopeangle = 0.01);

 //---------------------------------------------------------
 // small cluster minimum layer based track matching
 //---------------------------------------------------------
  virtual void TrackMinLayerMatching(S4CALSmallCluster* scls,
				     S4HYBTrack *trk,
				     Double_t distance = 15.);

 //---------------------------------------------------------
 // track matching for endcap region
 //---------------------------------------------------------
  virtual void TrackECMatching(S4CALSmallCluster* scls,
			       S4HYBTrack *trk,
			       Double_t cut = 10.); // (cm)

  //=================================================================
  //* Getter & Setter
  //-----------------------------------------------------------------

 public:
  inline    Int_t GetMinNHits() const { return fMinNHits;}
  inline Double_t GetEMOuterR() const { return fEMOuterR;}

 public:
  inline void SetMinNHits(Int_t nhits) { nhits > 4 ? fMinNHits = nhits : fMinNHits = 4;}
  inline void SetEMOuterR(Double_t r)  { fEMOuterR = r;}

  //=================================================================
  //* Data member
  //-----------------------------------------------------------------

 private:

  Int_t    fMinNHits;   // a small cluster has to have hits more than fMinNHits
  Double_t fEMOuterR;   // the outer radius of EM calorimeter


  ClassDef(S4CALTrackMatcher, 1)
};
    
#endif
