//*************************************************************************
//* ---------------------
//* S4CDCTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4CDCVectorHit"s for a single track.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCTrack		: Array containing "S4CDCVectorHit"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*     2003/0X/0X  Y.Nakashima  Modified for Kalman filter.
//*     2004/08/08  K.Fujii      Now inherits from U4CDCTrack.
//*************************************************************************

#include <iostream>
#include "JSFUtil.h"
#include "TMath.h"
#include "S4CDCTrack.h"
#include "S4CDCHitWire.h"
#include "S4CDCHit.h"
#include "JSFSteer.h"
#include "U4CDCKalHit.h"
#include "U4CDCMeasLayer.h"
#include "U4CDCKalDetector.h"
#include "TKalTrackState.h"
#include "TKalTrackSite.h"
#include "TKalTrack.h"

extern JSFSteer* gJSF;

//#define __DEBUG__

#define __BFIELD__       30.
#define __DRIFTVELOCITY__  669600  // cm/s, 0.9kV/cm
#define __DUMMYDXDRIFT__ 0.0085
#define __DUMMYDZA__     5.
#define __DUMMYDZS__     5.

ClassImp(S4CDCTrack)


//=====================================================================
//* constructor -------------------------------------------------------

S4CDCTrack::S4CDCTrack(S4CDCTrackMaker* trkm) 
                         : U4CDCTrack(trkm),
			   fMotherTrackPtr(0)
{
	SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CDCTrack::~S4CDCTrack()
{
}


//=====================================================================
//* IsSortable --------------------------------------------------------

Bool_t S4CDCTrack::IsSortable() const { return kTRUE; }

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4CDCTrack::Compare(const TObject *obj) const
{
	Int_t track  = GetTrackNumber();
	Int_t tracko = ((S4CDCTrack *)obj)->GetTrackNumber();
   	if (track < tracko) return -1;
   	else if (track > tracko) return 1;
   	else return 0;
}

//=====================================================================
//* Append -----------------------------------------------------------

Int_t S4CDCTrack::Append(S4CDCVectorHit *vectorhit)
{	
	Int_t track = vectorhit->GetTrackID();
	
	// Return value = -1:	new vectorhit.
	//		=  0:	successfully appended.
	//		=  1:	new track.
	
	if (GetEntries() == 0) {
		SetTrackNumber(track);
		vectorhit->SetTrackPtr(this);
                Add(vectorhit);
                return 0;
	} else {
		if (track == GetTrackNumber()) {
			vectorhit->SetTrackPtr(this);
                        Add(vectorhit);
                        return 0;
                } else {
                        return 1;
                }
        }
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4CDCTrack::DebugPrint(const Char_t *opt)
{
	std::cerr 	<< "track = " << GetTrackNumber() << std::endl;
	TIter next(this);
	S4CDCVectorHit *vectorhit;
	while ((vectorhit = (S4CDCVectorHit *)next())) {
		vectorhit->DebugPrint(opt);
	}
}
