#ifndef __S4CDCTRACKHIT__
#define __S4CDCTRACKHIT__

//*************************************************************************
//* ---------------------
//* S4CDCTrackHit Class
//* ---------------------
//* (Description)
//*	Class for a single track hit.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCTrackHit		: 
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

class S4CDCVectorHit;

#include "U4CDCTrackHit.h"
#include "S4CDCHit.h"
#include "S4CDCExactHit.h"
#include "S4CDCVectorHit.h"

class S4CDCTrackHit : public U4CDCTrackHit {
friend class S4CDCVectorHit;

public:

  S4CDCTrackHit(S4CDCExactHit* exacthitp);
  virtual ~S4CDCTrackHit(); 
  
  virtual void 		  DebugPrint(const Char_t *opt = "Brief");
  inline virtual void  	  Revive() ;
  inline virtual Bool_t          IsKilled() const;
  
    
  inline virtual S4CDCExactHit*  GetExactHitPtr();
  inline virtual Int_t           GetTrackID() const;
  inline virtual Int_t           GetLayerNumber() const;
  inline virtual Int_t           GetCellNumber () const;
  inline virtual Int_t           GetCellID     ();
  inline virtual Int_t           GetWireNumber () const;
  inline virtual Int_t           GetPDGCode    () const;
  
private:
  			    
  Bool_t          fKilled;		// flag to tell hits are killed or not.
  S4CDCExactHit  *fExactHitPtr;		// pointer to S4CDCExactHit.
  S4CDCVectorHit *fVectorHitPtr;	// pointer to S4CDCVectorHit.

  ClassDef(S4CDCTrackHit, 1) 	// CDC Hit Wire
};

//---------------------------------------
// inline function
//---------------------------------------

void S4CDCTrackHit::Revive() 
{
	fKilled = kFALSE;
}

Bool_t S4CDCTrackHit::IsKilled() const
{
	return fKilled;
}

S4CDCExactHit* S4CDCTrackHit::GetExactHitPtr()
{
	return fExactHitPtr;
}

Int_t S4CDCTrackHit::GetTrackID() const
{
	return fExactHitPtr->GetTrackID();
}

Int_t S4CDCTrackHit::GetLayerNumber() const
{
	return fExactHitPtr->GetLayerNumber();
}

Int_t S4CDCTrackHit::GetCellNumber() const
{
	return fExactHitPtr->GetCellNumber();
}

Int_t S4CDCTrackHit::GetCellID()
{
	return fExactHitPtr->GetCellID();
}

Int_t S4CDCTrackHit::GetWireNumber() const
{
	return const_cast<S4CDCTrackHit *>(this)->GetHitPtr()->GetWireNumber();
}

Int_t S4CDCTrackHit::GetPDGCode() const
{
        return fExactHitPtr->GetPDGCode();
}



#endif


