//*************************************************************************
//* ----------------
//* S4HYBTrack Class
//* ----------------
//* (Description)
//* (Description)
//*     Class to represent a (smeared) hybrid track.
//*     U4HYBTrack is an array of track segments.
//* (Requires)
//*     U4HYBTrack
//* (Provides)
//*     S4HYBTrack
//* (Update Record)
//*     2004/09/07  K.Fujii      Original version.
//*************************************************************************

#include <iostream>
#include "JSFUtil.h"
#include "TMath.h"
#include "U4VTrackMaker.h"
#include "S4HYBTrack.h"
#include "TVirtualPad.h"
#include "TKalTrackSite.h"

ClassImp(S4HYBTrack)

//=====================================================================
//* constructor -------------------------------------------------------

S4HYBTrack::S4HYBTrack(U4VTrackMaker* trkm) 
           : U4HYBTrack(trkm)
{
   SetOwner(kFALSE);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4HYBTrack::~S4HYBTrack()
{
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4HYBTrack::DebugPrint(const Char_t *opt)
{
   Update();

   using namespace std;
   cerr << "------------ S4HYBTrack::DebugPrint ------------" << endl
        << "   T0   = "  << GetTime()         << "  [nsec]"     << endl
        << "   Xv   = (" << GetPosition().X() << ", "
                         << GetPosition().Y() << ", "
                         << GetPosition().Z() << ") [cm]"       << endl
        << "   E    = "  << GetEnergy()       << "  [GeV]"      << endl
        << "   Pv   = (" << GetMomentum().X() << ", "          
                         << GetMomentum().Y() << ", "           
                         << GetMomentum().Z() << ") [GeV]"      << endl
        << "   Q    = "  << GetCharge()                         << endl
        << "   PID  = "  << GetPDGCode()                        << endl
#if 0
        << "   TKID = "  << GetTrackID()                        << endl;
#else
        << endl;
#endif

   if (opt[1] == 'e') { // "Detailed"
      TIter next(this);
      U4VTrack *vtrkp;
      while ((vtrkp = dynamic_cast<U4VTrack *>(next()))) {
         vtrkp->DebugPrint(opt);
      }
   }
}

//=====================================================================
//* GetTrackID --------------------------------------------------------

Int_t S4HYBTrack::GetTrackID()
{
   S4VTrack *vtkp = dynamic_cast<S4VTrack *>(((TObjArray *)this)->At(0));
   return vtkp->GetTrackID();
}

//=====================================================================
//* GetMotherTrackID --------------------------------------------------

Int_t S4HYBTrack::GetMotherTrackID()
{
   S4VTrack *vtkp = dynamic_cast<S4VTrack *>(((TObjArray *)this)->At(0));
   return vtkp->GetMotherTrackID();
}

//=====================================================================
//* GetPDGCode --------------------------------------------------------

Int_t S4HYBTrack::GetPDGCode()
{
   S4VTrack *vtkp = dynamic_cast<S4VTrack *>(((TObjArray *)this)->At(0));
   return vtkp->GetPDGCode();
}


//=====================================================================
//* Draw --------------------------------------------------------------

void S4HYBTrack::Draw(Int_t color, const Char_t *opt)
{
   if (!gPad || !GetEntries()) return;
                                                                                
   TIter next(this);
   TAttDrawable *objp = 0;
   while ((objp = dynamic_cast<TAttDrawable *>(next()))) {
      objp->Draw(color, opt);
   }
}


