#ifndef __S4ITPADHIT__
#define __S4ITPADHIT__
//*************************************************************************
//* ---------------------
//* S4ITHit Class
//* ---------------------
//* (Description)
//*	Class for a single IT hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	S4ITHit		: TObject containing single hit data
//* (Update Record)
//*	2004/09/06  A.Yamaguchi	Original version.
//*************************************************************************

#include "S4VExactHit.h"
#include "S4ITExactHit.h"
#include "U4ITHit.h"

class S4ITHit : public U4ITHit {

public:
  S4ITHit(S4ITExactHit * exacthit);
  
  //==========================================================
  // --- ATTENTION ------
  // Both copy constructor and operator= don't set "this" pointer
  // to their ExactHit's Smeared Hit Pointer.
  //
     inline       S4ITHit(const S4ITHit & right);
     inline const S4ITHit& operator=(const S4ITHit& right);
  //==========================================================
  
  virtual ~S4ITHit();

  // Getter --------------------------------------------------------------- 
  inline S4ITExactHit *GetExactHitPtr() const { return fExactHitPtr; }
  
  // Setter --------------------------------------------------------------- 
  static void SetSortBy(Char_t *opt = "TrackID");

  static void SetSigRPhi(Double_t sg=0.0){ fgDRPhi=sg; }
  static void SetSigZ   (Double_t sg=0.0){ fgDZ=sg; }
  static void SetSigX   (Double_t sg=0.0){ fgDX=sg; }
  static void SetSigY   (Double_t sg=0.0){ fgDY=sg; }

  // Other method --------------------------------------------------------- 
  virtual Int_t Compare   (const TObject *obj) const;
          void  DebugPrint(const Char_t * opt = "Brief");
          void  Update();  
  static  void Print();

private:
  virtual void   CalculatePosition();

private:
  static Int_t     fgSortBy;       // sort flag
  static Double_t  fgDRPhi;        // 
  static Double_t  fgDZ;           // 
  static Double_t  fgDX;           // 
  static Double_t  fgDY;           // 
  S4ITExactHit    *fExactHitPtr;   // Pointer to Exact Hit 

  ClassDef(S4ITHit, 1)          // IT Anode Hit
  
};

//=====================================================================
//---------------------------------
// inline function for S4ITHit
//---------------------------------

inline S4ITHit::S4ITHit(const S4ITHit& right)
{
   fExactHitPtr = right.fExactHitPtr;
   
   // !!! Don't set this to S4ITExactHit::SmearedHitPointer !!! 
}

inline const S4ITHit &S4ITHit::operator= (const S4ITHit& right)
{            
   fExactHitPtr = right.fExactHitPtr;
   
   // !!! Don't set this to S4ITExactHit::SmearedHitPointer !!! 
     return *this;
}

#endif
