//*************************************************************************
//* --------------------
//* S4ITTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/08/28  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4ITTrackMakerBuf.h"
#include "S4ITExactHit.h"
#include "S4ITTrack.h"
#include "S4TPCTrack.h"
#include "S4TPCTrackHit.h"
#include "S4ITTrackHit.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif
using namespace std;

//#define __DEBUG__

ClassImp(S4ITTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4ITTrackMakerBuf::S4ITTrackMakerBuf(const Char_t   *name,
                                     const Char_t   *title, 
                                     S4ITTrackMaker *module,
                                     Int_t           bufsize)
                 : U4ITTrackMakerBuf(name, title, module, bufsize)
{
}
       
S4ITTrackMakerBuf::S4ITTrackMakerBuf(S4ITTrackMaker *module,
                                     const Char_t   *name,
                                     const Char_t   *title, 
                                     Int_t           bufsize)
                 : U4ITTrackMakerBuf(module, name, title, bufsize)
{
#ifdef __DEBUG__
   cerr << "constructor for S4ITTrackMakerBuf is called..." << endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4ITTrackMakerBuf::~S4ITTrackMakerBuf()
{
}

//=====================================================================
//* MakeOneTrack ------------------------------------------------------

U4VTrack *S4ITTrackMakerBuf::MakeOneTrack(U4VTrack *htkp)
{
   if (!IsInitialized()) {
      cerr << "ERROR  S4ITTrackMakerBuf::MakeTracks" << endl
           << "...... hit data not yet initialized." << endl;
      abort();
   }
   TObjArray *inputbufp = dynamic_cast<S4ITHitMakerBuf *>
                          (GetInputDataBufPtr())->GetOutputBufPtr();

   S4ITHit::SetSortBy("Time");
   inputbufp->UnSort();
   inputbufp->Sort();
   S4ITTrack *trackp = new S4ITTrack(static_cast<S4ITTrackMaker *>(Module()));
   trackp->SetSeedTrackPtr(htkp);
   Int_t      hybID  = dynamic_cast<S4VTrack *>(htkp)->GetTrackID();

   TIter next(inputbufp);
   S4ITHit *hitp = 0;
   while ((hitp = static_cast<S4ITHit *>(next()))) {
      Int_t itID = hitp->GetExactHitPtr()->GetTrackID();
      if (itID == hybID) {
         S4ITTrackHit *thitp = new S4ITTrackHit(hitp);
         trackp->Add(thitp);
         if (hitp->IsLocked()) thitp->Lock();
      }
      if (trackp->GetTrackMakerPtr()->GetDetectorPtr()->GetNmeasPoints()
       == trackp->GetEntries()) break;
   }

   if (!trackp->Verify()) {
      delete trackp;
      return 0;
   }

   GetOutputBufPtr()->Add(trackp);
   return trackp;
}

//=====================================================================
//* //DebugPrint --------------------------------------------------------

void S4ITTrackMakerBuf::DebugPrint(const Char_t * /*opt*/)
{
}

