//*****************************************************************************
//* ==============
//* S4VHit
//* ==============
//*   Base class for METIS (smeared) hits.
//*
//* The following virtual functions must be implemented in a derived class:
//*   IsSortable()
//*     ---> Simply returns a Bool_t for a sortable hit.
//*   Compare(const TObject *obj)
//*     ---> Compare a hit object with this hit. Return the following values:
//*           -1, this < hit
//*            0, this = hit
//*           +1, this > hit
//*
//* (Update Record)
//*   2004.07.30  A.Sanchez  Primitive version.
//*****************************************************************************

#include "S4VHit.h"

ClassImp(S4VHit)

S4VHit::ESortOption S4VHit::fSortBy = kEnergy;

//==================================================================
//* constructor ----------------------------------------------------

S4VHit::S4VHit()
      : fHitPosition(0,0,0), fEnergyDeposit(0), fTOF(0), 
        fAddress(0), fExactHits(0)
{
}

S4VHit::S4VHit(Double_t      edep, 
               Double_t      tof, 
         const TVector3    & pos,
               TVAddress   * addr,
               S4VExactHit * hit)
      : fHitPosition(pos), fEnergyDeposit(edep), fTOF(tof), 
        fAddress(addr), fExactHits(0)
{
  if (hit) {
     AddExactHit(hit);
  }
}

//==================================================================
//* destructor -----------------------------------------------------

S4VHit::~S4VHit()
{
  if (fExactHits) delete fExactHits;
}

//==================================================================
//* Add ------------------------------------------------------------
void S4VHit::AddExactHit(S4VExactHit *hit)
{
   if (!fExactHits) fExactHits = new TObjArray();
   fExactHits->Add(hit);
}
