#ifndef __S4VTRACK__
#define __S4VTRACK__
//*****************************************************************************
//* ==============
//* class S4VTrack
//* ==============
//* (Purpose)
//*    Attribute class for METIS (smeared) track.
//*    Users must implement the following pure virtual functions:
//*       GetTrackID ()
//*       GetPDGCode ()
//* (Dependency)
//*    Requires: TVector3
//*    Provides: S4VTrack
//* (Update Record)
//*   2004/09/05  K.Fujii   Orivinal very primitive version.
//*****************************************************************************

#include "TVector3.h"

class S4VTrack {
public:
  // ==========================================================================
  // ------------------
  //  Member Functions
  // ------------------

  S4VTrack();
  virtual ~S4VTrack();

  // --------------------------------------------------------------------------
  // -------------------
  //  Sorting functions
  // -------------------
  enum ESortOption_t { kTrackID,
                       kTime,
                       kEnergy };

#if 0
  virtual Bool_t  IsSortable() const { return kTRUE; }
  virtual Int_t   Compare(const TObject *obj) const = 0;
#endif

  static void     SetSortBy(ESortOption_t key = kTrackID) { fgSortBy = key; }
  static Int_t    GetSortBy() { return (Int_t)fgSortBy; }

  // --------------------------------------------------------------------------
  // --------------
  //  Data Getters
  // --------------
  virtual Double_t    GetTime    (Double_t d = 0.);  // time     -+ 
  virtual TVector3    GetPosition(Double_t d = 0.);  // position  |
  virtual Double_t    GetEnergy  (Double_t d = 0.);  // energy    +- at dphi = d
  virtual TVector3    GetMomentum(Double_t d = 0.);  // momentum -+

  virtual Int_t       GetTrackID       () = 0;       // track ID from Jupiter
  virtual Int_t       GetMotherTrackID () = 0;       // mom track ID from Jupiter
  virtual Int_t       GetPDGCode       () = 0;       // PDG code
  virtual Double_t    GetCharge        ();           // charge    

private:
  virtual void CalcStateAt(Double_t d = 0.); // calculates state at dphi = d

  // ==========================================================================
  // --------------
  //  Data Members
  // --------------
private:
  Double_t     fT;                 //  last T
  TVector3     fXv;                //  last Xv
  Double_t     fE;                 //  last E
  TVector3     fPv;                //  last Pv
  Double_t     fDphiLast;          //  last dphi
  Double_t     fCharge;            //  charge

  static ESortOption_t fgSortBy;    //! sort key

  ClassDef(S4VTrack, 1) // a (smeared) track base class
};

#endif
