//*************************************************************************
//* ---------------------
//* S4VTXTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4VTXTrackHit"s for a single track.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4VTXTrack		: Array containing "S4VTXTrackHit"s
//* (Update Record)
//*	2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "S4VTXTrack.h"
#include "S4VTXTrackHit.h"
#include "TPolyMarker3D.h"
#include "TVirtualPad.h"
//#define __DEBUG__

ClassImp(S4VTXTrack)

//=====================================================================
//* constructor -------------------------------------------------------

S4VTXTrack::S4VTXTrack()
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4VTXTrack is called..." << std::endl;
#endif
   SetOwner();
}

S4VTXTrack::S4VTXTrack(S4VTXTrackMaker* trkm)
          : U4VTXTrack(trkm)
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4VTXTrack is called..." << std::endl;
#endif
   SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4VTXTrack::~S4VTXTrack()
{
}

//=====================================================================
//* GetPDGCode --------------------------------------------------------

Int_t S4VTXTrack::GetPDGCode()
{
   S4VTXTrackHit *thp = (S4VTXTrackHit *)((TObjArray *)this)->operator[](0);
   S4VTXHit   *php = (S4VTXHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetPDGCode();
}

//=====================================================================
//* GetTrackID --------------------------------------------------------

Int_t S4VTXTrack::GetTrackID()
{
   S4VTXTrackHit *thp = (S4VTXTrackHit *)((TObjArray *)this)->operator[](0);
   S4VTXHit   *php = (S4VTXHit *)thp->GetHitPtr();
#if 0
   std::cerr << "tptrk = " << (void *)this << " j4trkid = " << php->GetExactHitPtr()->GetTrackID() << std::endl;
#endif

   return php->GetExactHitPtr()->GetTrackID();
}

//=====================================================================
//* GetMotherTrackID --------------------------------------------------

Int_t S4VTXTrack::GetMotherTrackID()
{
   S4VTXTrackHit *thp = (S4VTXTrackHit *)((TObjArray *)this)->operator[](0);
   S4VTXHit   *php = (S4VTXHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetMotherTrackID();
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4VTXTrack::Draw(Int_t color, const Char_t * /* opt */)
{
   if (!gPad) return;
   gPad->cd();

   TPolyMarker3D *pm3dp = new TPolyMarker3D(this->GetEntries());
   pm3dp->SetBit(kCanDelete);
   pm3dp->SetMarkerColor(color);

   Int_t nhits = 0;
   TIter next(this);
   S4VTXTrackHit *trkhitp = 0;
   while ((trkhitp = static_cast<S4VTXTrackHit *>(next()))) { 
      TVector3 pos = trkhitp->GetHitPtr()->GetExactHitPtr()->GetPosition();
      pm3dp->SetPoint(nhits, pos.X(), pos.Y(), pos.Z());
      nhits++;
   }
   pm3dp->Draw();
   gPad->Update();
}

//=====================================================================
//* Verify ------------------------------------------------------------

Bool_t S4VTXTrack::Verify()
{
   if (!GetEntries()) return kFALSE;

   S4VTXTrackHit::SetSortBy("Time");
   UnSort(); 
   Sort(); 

   if (GetEntries() > 1) {
      TIter next(this);
      Int_t lyr1 = static_cast<S4VTXTrackHit *>
                  (next())->GetHitPtr()->GetExactHitPtr()->GetLayerID();
      Int_t lyr2 = static_cast<S4VTXTrackHit *>
                  (next())->GetHitPtr()->GetExactHitPtr()->GetLayerID();
      if (lyr2 < lyr1) return kFALSE;
      next.Reset();

      S4VTXTrackHit *thitp = 0;
      Int_t last = -999;
      while ((thitp = static_cast<S4VTXTrackHit *>(next()))) {
         S4VTXExactHit *ehitp = thitp->GetHitPtr()->GetExactHitPtr();
         Int_t cur = ehitp->GetLayerID();
         if (cur <= last) {
            thitp->Lock();
            break;
         }
         last = cur;
      }
      while ((thitp = static_cast<S4VTXTrackHit *>(next()))) thitp->Lock();
   }
   return kTRUE;
}

