//
// cdcanl.C
//
// 

#include <iomanip>

void cdcanl(Int_t run = 0)
{
//----------------------------------
// Reset ROOT
//----------------------------------

   gROOT->Reset();

//----------------------------------
// Load user defined classes
//----------------------------------

   gSystem->Load("libS4Utils.so");
   gSystem->Load("libS4MCTruth.so");
   gSystem->Load("libS4VTXMCTruth.so");
   gSystem->Load("libS4CALMCTruth.so");
   gSystem->Load("libS4ITMCTruth.so");
   gSystem->Load("libS4CTMCTruth.so");
   gSystem->Load("libS4TPCMCTruth.so");
   gSystem->Load("libS4CDCMCTruth.so");
   gSystem->Load("libJSFJ4.so");

   gSystem->Load("libS4Kalman.so");  
   gSystem->Load("libS4Geom.so");  
   gSystem->Load("libS4KalTrack.so");  
   gSystem->Load("libU4CDCRawData.so");
   gSystem->Load("libU4CDCClusterMaker.so");
   gSystem->Load("libU4VHitMaker.so");
   gSystem->Load("libU4KalDetector.so");  
   gSystem->Load("libU4CDCKalDetector.so");  
   gSystem->Load("libU4CDCHitMaker.so");
   gSystem->Load("libU4VTrackMaker.so");  
   gSystem->Load("libS4Kern.so");
   gSystem->Load("libS4CDCHitMaker.so");
   gSystem->Load("libU4CDCTrackMaker.so");  
   gSystem->Load("libS4CDCTrackMaker.so");  
   gSystem->Load("libU4HYBTrackMaker.so");
   gSystem->Load("libS4HYBTrackMaker.so");
   gSystem->Load("libS4CDCAnalysis.so");

   gErrorAbortLevel = kWarning + 1; 

//----------------------------------
// Create JSF runtime environment
//----------------------------------

   JSFSteer *jsf = new JSFSteer();
  
//----------------------------------
// Set output file
//----------------------------------

   TFile *foutp = new TFile("jsf.root","RECREATE"); //  Output file.
   TFile &fout  = *foutp;

   jsf->SetOutput(fout);

//----------------------------------
// Set input file
//----------------------------------

   if (!run) run = jsf->Env()->GetValue("JSFSteer.RunNo",1615);  
   cerr << " RunNo = " << run << endl;

   Int_t nevent = jsf->Env()->GetValue("JSFSteer.Nevent",98);  
   Int_t minevt = 0;
   Int_t maxevt = minevt + nevent;

   S4VMCTruth::SetInputFileName("hit.dat");

//----------------------------------
// Define KalDetector
//----------------------------------
#if 1
   // Axial Only Mode (should set before creating U4CDCKalDetector)
   //===================================================================
                 U4CDCKalDetector::SetAxialOnly();
   //===================================================================
#endif
   U4CDCKalDetector *cdcdet      = new U4CDCKalDetector();

//----------------------------------
// Insert modules
//----------------------------------

   S4CDCMCTruth    *mctruth     = new S4CDCMCTruth();  
   S4CDCHitMaker   *hitmaker    = new S4CDCHitMaker();
   S4CDCTrackMaker *trackmaker  = new S4CDCTrackMaker(cdcdet);
   S4HYBTrackMaker *htrackmaker = new S4HYBTrackMaker(cdcdet);
   S4CDCAnalysis   *analysis    = new S4CDCAnalysis();

//----------------------------------
// Set parameters
//----------------------------------

   // Remove Dalta-Ray hits ....    
#if 0
      mctruth->RemoveDeltaRayHits(kFALSE);
      mctruth->JoinSeparatedHits(kFALSE);
#else
      mctruth->RemoveDeltaRayHits(kTRUE);
      mctruth->JoinSeparatedHits(kTRUE);
#endif

   // Saturation effect ....    
      analysis->SaturationOff();
 
   // Two-track Separation ....
      Int_t twotrack = jsf->Env()->GetValue("JSFSteer.TwoTrackSeparation",2.0);  
      Double_t twotracksep = twotrack/10.; // [cm]
      S4CDCHitWire::SetTwoHitSeparation(twotracksep);
      cerr << " twotracksep = " << twotrack << "[mm]" << endl;  
#if 0
      S4CDCHitWire::SetTwoHitSeparationZ(0.3); // [cm]
#endif
#if 0
      S4CDCHit::SetSpatialResolution("experimental"); // default is "flat".
#endif
 
   // Set IP constraint ....
#if 0
      S4CDCTrack::SetIPConstraint(kTRUE);
#endif

   // Artificial bunch tagging
      Double_t t0 = 14.;
      S4CDCTrack::SetT0pedestal(t0); // [nsec]
     
   // Simulation Mode (CDC or TPC) ....
      S4CDCHitWire::TPCMode(kFALSE);

//----------------------------------
// Job initialization
//----------------------------------
 
   jsf->Initialize();

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  htrackmaker->SetMainTrackMakerPtr(trackmaker);
  //htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  //htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);
    
   jsf->BeginRun(run);      // Set run number

//----------------------------------
// Start event loop
//----------------------------------

   for(Int_t ev = minevt; ev < maxevt; ev++) {
      Int_t ib;
      if (!jsf->Process(ev)) {
         cerr << "jsf->Process is faild." << endl;
         jsf->FillTree();
         jsf->Clear();
         break;
      }
      jsf->FillTree();
      jsf->Clear();
   }
   jsf->Terminate();
   fout.Write();
}
