//  
// cdcanl.C
//
// 

TFile *foutp;
TH1F  *hTrunc;
TH2F  *hGpibhodo;


int cdcanl(Int_t run = 0)
{
  gROOT->Reset();
  
  gSystem->Load("libS4Utils.so");
  gSystem->Load("libS4Kalman.so");  
  gSystem->Load("libS4Helix.so");  
  gSystem->Load("libS4KalTrack.so");
  gSystem->Load("libU4CDCRawData.so");
  gSystem->Load("libU4CDCClusterMaker.so");
  gSystem->Load("libU4VHitMaker.so");
  gSystem->Load("libU4CDCHitMaker.so");
  gSystem->Load("libU4VTrackMaker.so");
  gSystem->Load("libU4CDCKalDetector.so");
  gSystem->Load("libU4CDCTrackMaker.so");
  gSystem->Load("libU4BabyTrackMaker.so");
  gSystem->Load("libU4VAnalysis.so");
  gSystem->Load("libU4BabyAnalysis.so");

#if 0
// For 98 data
  U4CDCCluster::SetNpres(20);
  U4CDCCluster::SetNposts(20);
  U4CDCUniversalRecordHeader::UseOldFormat();
#endif
    
  JSFSteer *jsf = new JSFSteer();

  if (!run) run = jsf->Env()->GetValue("JSFSteer.RunNo",3039);  
  Char_t foname[64];
  sprintf(foname,"./jsf%4.4dconstped.root",run);
  foutp = new TFile(foname,"RECREATE"); //  Output file.
  TFile & fout = *foutp;
  
  Char_t fname[64];
  // sprintf(fname,"run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2000/beam/unpacked/run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2000/dedx/unpacked/run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2001/beam/unpacked/run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data2/baby/2001/beam/unpacked/run%4.4d.root",run);
  sprintf(fname,"run%4.4d.root",run);
  TFile file(fname); // Input file

  jsf->SetInput(file);
  jsf->SetOutput(fout);

  // TTree::SetBranchStyle(0);

  TDirectory *last = gDirectory;
  foutp->cd("/");
  hTrunc = new TH1F("Trunc","truncated mean",100,0.,8000.);
  hGpibhodo = new TH2F("Gpibhodo","counter vs current",100,0.,5000,1000,0.,0.005);

  last->cd();

  Int_t nevent=jsf->Env()->GetValue("JSFSteer.Nevent",100000);  
  Int_t minevt=1;
  Int_t maxevt=minevt+nevent;

  cout << " Run = " << run << " Nevents = " << nevent << endl;

  U4CDCFADCCalibration fadccalib("../../../../data/fadccalib.dat");
  U4CDCClusterMaker *clust = new U4CDCClusterMaker(&fadccalib);
#if 1
  clust->UseConstPed();
#else
  clust->UseVarPed();
#endif
#if 1
  U4CDCCluster::SetSoftThresh(16.);
  U4CDCCluster::SetSplitThresh(20.);
#else
  U4CDCCluster::SetSoftThresh(20.);
  U4CDCCluster::SetSplitThresh(40.);
#endif
#if 1
  U4CDCCluster::SetMinNslots(20); // for cluster splitting
#else
  U4CDCCluster::SetMinNslots(100); // for cluster splitting
#endif
  U4CDCCluster::SetMaxNslots(250); // for ph sum
  U4CDCCluster::SetIntegrateNslots(20); // for smoothing

  U4CDCTimeToXRelation xtcalib("../../../../data/xtcalib.dat");
#if 1
  U4CDCXTDatum::SetSigmaMS(0.000);
#else
  U4CDCXTDatum::SetSigmaMS(0.050);
#endif

  U4CDCHitMaker   *hit   = new U4CDCHitMaker(&xtcalib);
  U4BabyTrackMaker *track = new U4BabyTrackMaker();

#if 0
  U4CDCRawHist      *hist  = new U4CDCRawHist();
#endif
#if 0
  U4CDCClusterHist  *hist  = new U4CDCClusterHist();
#endif
#if 1
  U4BabyAnalysis     *hist  = new U4BabyAnalysis();

  hist->SetMode(0);		// (0,1) = (excl,incl) hit in question from fit
  hist->SetNwires(5);		// require Nwires or more in the analysis
  hist->SetChi2Cut(15.);	// require chi2 < xChi2Cut for reference track
  hist->SetNsigCut(5.);  	// window for residals
  hist->SetTrackRange(0,999);   // (i1,i2) = (1st,last) tracks to look at
                                // (0,0) to look at track closest to wire
                                // (1,1) to study 2-track separation 
  hist->SetSclHodSum(5000);	// hodsum cut to separate on/off spill
  hist->SetX7Lo(1.0);		// lower cut on X at 7cm
  hist->SetX7Hi(4.0);		// higher cut on X at 7cm
#endif

  jsf->Initialize();
 
  // EventBuf set only after Initialize()!

#if 1
  U4BabyTrackMakerBuf *tkb = (U4BabyTrackMakerBuf *)track->EventBuf();
#if 1
  tkb->SetNsigCut(5.);  	// window for residals
  tkb->SetChi2Cut(15.); 	// chi2 cut for tracks
  tkb->SetKCNsigCut(3.); 	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05); 	// DX to be qualified as a pivot
  tkb->SetSlopeCut(0.15); 	// cut on track slope
#else
  tkb->SetNsigCut(10.); 	// window for residals
  tkb->SetChi2Cut(45.); 	// chi2 cut for tracks
  tkb->SetKCNsigCut(2.); 	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05); 	// DX to be qualified as a pivot
  tkb->SetSlopeCut(0.15); 	// cut on track slope
#endif

  U4CDCHitMakerBuf *htb = (U4CDCHitMakerBuf *)hit->EventBuf();
#if 1
  htb->SetWidthCut(12.);
  htb->SetPeakCut(30.);
  htb->SetSumCut(200.);
#else
  htb->SetWidthCut(60.);
  htb->SetPeakCut(30.);
  htb->SetSumCut(1600.);
#endif
#endif

  U4CDCRawData *raw=(U4CDCRawData*)jsf->FindModule("U4CDCRawData");

cerr << "break 1" << endl;
  jsf->BeginRun(run);	// Set run number
cerr << "break 2" << endl;
  

  for (Int_t ev = minevt; ev <= maxevt; ev++) {
cerr << "ev =" << ev << endl;
    Int_t ib;
    if( ! (ib=jsf->GetEvent(ev)) ) break ;
cerr << "break 3" << endl;
#if 1 
    ((U4CDCRawDataBuf*)raw->EventBuf())->GetHeader()->DebugPrint();
#endif
    if( ! jsf->Process(ev) ) break ;
#if 1
    ((U4CDCClusterBuf*)clust->EventBuf())->DebugPrint();
    ((U4CDCHitMakerBuf*)hit->EventBuf())->DebugPrint();
    ((U4BabyTrackMakerBuf*)track->EventBuf())->DebugPrint("Detailed");
#endif

    //   Int_t sclhodx[5], sclhody[3];
    //   Int_t scltc1, scltc2, sclche, scltof1, scltof2;
    //   Int_t sclpi, sclp, scle, sclnopid;
    //GetScalerInfo(sclhodx,sclhody,scltc1,scltc2,sclche,scltof1,scltof2,
    //             sclpi,sclp,scle,sclnopid);


   //      Int_t sclhodsum = 0;
   //   for (Int_t ix = 0; ix < 5; ix++) sclhodsum += sclhodx[ix];
   //   for (Int_t ix = 0; ix < 3; ix++) sclhodsum += sclhody[ix];

#if 1
    Double_t gpicurr;
    Double_t gpib = hist->GetGPIBInfo(gpicurr);
    
    Int_t sclhodx[5], sclhody[3];
    Int_t scltc1, scltc2, sclche, scltof1, scltof2;
    Int_t sclpi, sclp, scle, sclnopid;
    

    Double_t count= hist->GetScalerInfo(sclhodx,sclhody,scltc1,scltc2,sclche,scltof1,scltof2,sclpi,sclp,scle,sclnopid);
          Int_t sclhodsum = 0;
      for (Int_t ix = 0; ix < 5; ix++) sclhodsum += sclhodx[ix];
      for (Int_t ix = 0; ix < 3; ix++) sclhodsum += sclhody[ix];
      cerr << "sclhodsum = " << sclhodsum << "  gpib = " << gpicurr<< endl;
    hGpibhodo->Fill(sclhodsum,gpicurr,1.0);
#endif

    

    jsf->FillTree();
    jsf->Clear();
  }

  jsf->Terminate();
  fout.Write();
}


