{
//*----------------------------------------------------------------------------
//* ================
//*  Macro plotall.C
//* ================
//*
//* (Description)
//*	A sample macro to display plots created by cdcanl.C.
//* (Usage)
//*	$ jsf -b -q unpack.C	# unpack rawdata and produce runxxx.root.
//*	$ jsf -b -q cdcanl.C	# to analyze runxxx.root.
//*	$ jsf jsf.root		# start jsf in interactive mode
//*	......
//*	root [1] .x plotall.C
//*
//*	This will display residual plots and opens an additional empty
//*	window as a temporary work canvas.
//*
//* (Update Record)
//*	2000/05/20	K.Fujii		Original version.
//*
//*----------------------------------------------------------------------------

gROOT->Reset();

Int_t nZoneX = 5;	// corresponds to # wires
Int_t nZoneY = 4;	// corresponds to # plots/wire 

JSFSteer *jsf = new JSFSteer();
Int_t run = jsf->Env()->GetValue("JSFSteer.RunNo",1034);

Char_t fname[64];
sprintf(fname,"jsf%4.4dconstped.root",run);
//sprintf(fname,"jsf.root",run);
TFile file(fname); // Input file


jsf->SetInput(file);         


TCanvas *c1 = new TCanvas("c1","DX-X",20,20,200*nZoneX,200*nZoneY+100);
c1->SetHighLightColor(5);
c1->SetFillColor(19);

TPaveLabel *pl = new TPaveLabel(0.0130952,0.956284,0.372619,0.989071,fname,"br");
pl->SetFillColor(18);
pl->SetTextSize(0.99);
pl->Draw();

TPad* p1 = new TPad("p1", fname, 0, 0, 1, 0.95); 
p1->Draw(); 
p1->cd();
p1->Divide(nZoneX, nZoneY);


//TCanvas *c2 = new TCanvas("c2","Work",20,50+200*nZoneY,480,380);

const Int_t ncmds = 5;
Char_t *template[] = {
	"p1->cd(id++);\
	TH2F *h2 = PhPk%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",
	
	"p1->cd(id++);\
	TH2F *h2 = Ph%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",
	
	"p1->cd(id++);\
	TH2F *h2 = Res%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",

	"TH2F *h2 = Res%2.2d;\
	h2->ProjectionY();\n",

	"p1->cd(id++);\
	TH1D *h1 = Res%2.2d_py;\
	h1->SetFillColor(42);\
	h1->Fit(\"gaus\");\n"
};

gStyle->SetOptFit();	// to display fit parameters in each window

Int_t id = 1;	// directory counter
Int_t j;
for (j = 0; j < ncmds; j++) {
	Int_t i;
	for (i = 0; i < nZoneX; i++) {
		Char_t command[256];
		sprintf(command,template[j],i,i);
		gROOT->ProcessLine(command);
		p1->Update();
	}
}
//c2->cd();
Char_t psname[64];
sprintf(psname,"jsf%4.4d.plotall.ps",run);
c1->Print(psname);
}


