#ifndef __U4BabyTRACK__
#define __U4BabyTRACK__

//*************************************************************************
//* ---------------------
//* U4BabyTrack Class
//* ---------------------
//* (Description)
//*	Class to store "U4BabyHitCell"s for a single layer.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4BabyTrack		: Array containing "U4BabyHitCell"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima  Original version.
//*	2004/08/08  K.Fujii      Modified according to U4CDCTrack changes.
//*************************************************************************

#include "U4CDCTrack.h"
#include "U4CDCVectorHit.h"
#include "JSFHelicalTrack.h"
#include "U4CDCTrackHit.h"

class U4CDCVectorHit;

class U4BabyTrack : public U4CDCTrack {
public:
  U4BabyTrack();
  U4BabyTrack(const U4BabyTrack &tk);
  virtual ~U4BabyTrack();

         static  void         SetSortBy(Int_t key) { U4VTrack::SetSortBy(key); }
         static  void         SetSortBy(Char_t *opt, Int_t wireno);
         virtual Int_t        Compare(const TObject *obj) const;

  inline virtual Double_t     Xat(Double_t y);
  inline virtual Double_t     SGX2at(Double_t y); 
  inline virtual Double_t     GetSlope();
  inline virtual Double_t     GetIntercept();

  inline virtual void         Update() 
  { 
     if (!IsDone()) {
        SetDone();
        Fit(); 
     }
  }

  virtual void 		DebugPrint(const Char_t *opt = "Brief");

private:
  virtual void		Fit();

private:         
  Double_t         fa;                     // a when x = a*y + b
  Double_t         fb;                     // b
  Double_t         fEaa;                   // <da*da>
  Double_t         fEba;                   // <db*da>
  Double_t         fEbb;                   // <db*db>
  static Int_t     fWireToLookAt;          // wire No. to look at
  static Int_t     fCellToLookAt;          // cell No. to look at

  ClassDef(U4BabyTrack, 1)		   // Baby Track
};

//-----------------------------------------------------------
//  inline function
//-----------------------------------------------------------

Double_t U4BabyTrack::Xat(Double_t y)
{
   if (!IsDone()) Update();
   return (fa*y + fb);
}
Double_t U4BabyTrack::SGX2at(Double_t y)
{ 
   if (!IsDone()) Update();    
   return (fEaa*y*y + 2*fEba*y + fEbb);
}
Double_t U4BabyTrack::GetSlope()
{
   if (!IsDone()) Update();
   return fa;
}
Double_t U4BabyTrack::GetIntercept()
{
   if (!IsDone()) Update();
   return fb;
}

#endif
