#ifndef __U4CDCTIMETOXRELATION__
#define __U4CDCTIMETOXRELATION__

//************************************************************************
//* -----------------------------
//* U4CDCTimeToXRelation Class
//* -----------------------------
//************************************************************************

#include "U4CDCHit.h"
#include "U4CDCRawData.h"

// ---------------------
// X-t Calibration Datum
// ---------------------

class U4CDCXTDatum : public TObject {
public:
  U4CDCXTDatum(Int_t n = 0, Double_t *x = 0, Double_t *t =0);
  U4CDCXTDatum(U4CDCXTDatum &xt);
  ~U4CDCXTDatum();

  void SetDXpar(Int_t n, Double_t *par);
  static void SetSigmaMS(Double_t sigma);

  Int_t WhereIs(Double_t t);
  Double_t X(Double_t t);
  Double_t DX(Double_t x);
  Double_t Vd(Double_t t);

  void     TimeToHitInfo(Double_t t, Double_t &x, Double_t &dx, Double_t &vd);

private:
  Int_t     fN;			//! No. data points
  Double_t *fX;			//! x (cm)
  Double_t *fT;			//! t (nsec)
  Int_t     fNpar;		//! No. parameters for DX function
  Double_t *fDXpar;		//! parameters for DX function

  static Double_t fSigmaMS;		//! temporary fudge factor include MS effects

  ClassDef(U4CDCXTDatum, 1)		// CDC X-T datum
};

// --------------------
// X-t Calibration Data
// --------------------

class U4CDCTimeToXRelation : public TObjArray {
private:
  class CalibDatum : public TObjArray {
     public:
	CalibDatum(Int_t last = 999999) : TObjArray(5), fLastRun(last) {}
	~CalibDatum() { if (GetEntries()) Delete(); }

	Int_t GetLastRun() { return fLastRun; }
	Double_t X(U4CDCCluster *fClusterPtr)
	{
		Int_t n = fClusterPtr->GetStationNo();
		Int_t a = fClusterPtr->GetChannelNo();
		Int_t w = ChannelToWire(n,a);
		Double_t t = fClusterPtr->GetStartTime();
		return ((U4CDCXTDatum *)At(w))->X(t);
	}
	Double_t DX(U4CDCCluster *fClusterPtr)
	{
		Int_t n = fClusterPtr->GetStationNo();
		Int_t a = fClusterPtr->GetChannelNo();
		Int_t w = ChannelToWire(n,a);
		return ((U4CDCXTDatum *)At(w))->DX(X(fClusterPtr));
	}
	Double_t Vd(U4CDCCluster *fClusterPtr)
	{
		Int_t n = fClusterPtr->GetStationNo();
		Int_t a = fClusterPtr->GetChannelNo();
		Int_t w = ChannelToWire(n,a);
		Double_t t = fClusterPtr->GetStartTime();
		return ((U4CDCXTDatum *)At(w))->Vd(t);
	}
        void TimeToHitInfo(U4CDCCluster *fClusterPtr,
                           Double_t &x, Double_t &dx, Double_t &vd)
        {
		Int_t n = fClusterPtr->GetStationNo();
		Int_t a = fClusterPtr->GetChannelNo();
		Int_t w = ChannelToWire(n,a);
		Double_t t = fClusterPtr->GetStartTime();
		((U4CDCXTDatum *)At(w))->TimeToHitInfo(t,x,dx,vd);
        }
     private:
	Int_t ChannelToWire(Int_t n, Int_t a)
        {
           U4CDCRawData     *rd = (U4CDCRawData *)gJSF->FindModule("U4CDCRawData");
           U4CDCBeginRunBuf *br = rd->GetBeginRunBuf();
           Int_t id, w;
           Int_t cr = 0;        // Only one crate at the moment.
           br->EChannelToDChannel(cr,n,a,id,w);
           return w;
        }
	Int_t	fLastRun;	//! Last Run Number
  };

public:
//  U4CDCTimeToXRelation() : TObjArray(1) {TObject tmp; Add(&tmp);}
  U4CDCTimeToXRelation(Int_t n = 0, U4CDCXTDatum *xt = 0, Int_t last = 999999);
  U4CDCTimeToXRelation(Char_t * calibfile);
  
  virtual ~U4CDCTimeToXRelation(); 
  Double_t X (U4CDCCluster *fClusterPtr) { return fCurr->X(fClusterPtr);  }
  Double_t DX(U4CDCCluster *fClusterPtr) { return fCurr->DX(fClusterPtr); }
  Double_t Vd(U4CDCCluster *fClusterPtr) { return fCurr->Vd(fClusterPtr); }

  void TimeToHitInfo(U4CDCCluster *fClusterPtr, Double_t &x, Double_t &dx, Double_t &vd)
  { 
     return fCurr->TimeToHitInfo(fClusterPtr, x, dx, vd);
  }
  
  void Update();

private:
  CalibDatum *fCurr;	//! pointer to current calib datum

  ClassDef(U4CDCTimeToXRelation, 1)	// CDC Hit Calibration
};
#endif
