//*************************************************************************
//* ===================
//*  U4IPMeasLayer Class
//* ===================
//*
//* (Description)
//*   Sample measurement layer class used by U4IPKalHit.
//* (Requires)
//* (Provides)
//*     class U4IPMeasLayer
//* (Update Recored)
//*   2003/09/30  Y.Nakashima       Original version.
//*
//*************************************************************************

#include "U4IPMeasLayer.h"
#include "U4IPKalHit.h"

ClassImp(U4IPMeasLayer)

//=====================================================================
//* constructor -------------------------------------------------------

U4IPMeasLayer::U4IPMeasLayer(TMaterial &matIn,
                             TMaterial &matOut,
                             TVector3  &xc,
                             TVector3  &n,
                             Bool_t     isactive)
              : U4VMeasLayer(matIn, matOut, isactive),
                TPlane(xc, n)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4IPMeasLayer::~U4IPMeasLayer()
{
}
                                                                                
//=====================================================================
//* XvToMv ------------------------------------------------------------

TKalMatrix U4IPMeasLayer::XvToMv(const TVector3 &xv) const
{
   // Calculate hit coordinate information:
   //	mv(0,0) = x
   //     (1,0) = z

   TKalMatrix mv(kMdim,1);
   mv(0,0)  = xv.X();
   mv(1,0)  = xv.Z(); 
   return mv;
}

TKalMatrix U4IPMeasLayer::XvToMv(const TVTrackHit &,
                                 const TVector3   &xv) const
{
   return XvToMv(xv);
}

//=====================================================================
//* HitToXv -----------------------------------------------------------

TVector3 U4IPMeasLayer::HitToXv(const TVTrackHit &vht) const
{
   const U4IPKalHit &ht = *dynamic_cast<const U4IPKalHit *> (&vht);

   Double_t z   = ht(1,0);
   Double_t x   = ht(0,0);
   Double_t y   = 0.;

   return TVector3(x,y,z);
}

//=====================================================================
//* CalcDhDa ----------------------------------------------------------

void U4IPMeasLayer::CalcDhDa(const TVTrackHit &vht,
                             const TVector3   &xxv,
                             const TKalMatrix &dxphiada,
                                   TKalMatrix &H)  const
{
   // Calculate
   //    H = (@h/@a) = (@phi/@a, @z/@a)^t
   // where
   //        h(a) = (phi, z)^t: expected meas vector
   //        a = (drho, phi0, kappa, dz, tanl, t0)
   //

   Int_t sdim = H.GetNcols();
   Int_t hdim = TMath::Max(5,sdim-1);

   // Set H = (@h/@a) = (@d/@a, @z/@a)^t
   
   for (Int_t i=0; i<hdim; i++) {
      H(0,i) = dxphiada(0,i);
      H(1,i) = dxphiada(2,i);
   }
   if (sdim == 6) {
      H(0,sdim-1) = 0.;
      H(1,sdim-1) = 0.;
   }
}

