#ifndef __U4TPCHIT__
#define __U4TPCHIT__
//*************************************************************************
//* ---------------------
//* U4TPCHit Class
//* ---------------------
//* (Description)
//*	Class for a single TPC hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4TPCHit		: TObject containing single hit data
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "TAttLockable.h"

class U4TPCHit : public TObject, public TAttLockable {

public:
  U4TPCHit(Int_t    padrowno = -999,
           Double_t padrowr  = -999.,
           Int_t    side     = -999,
           Double_t phi      = -999.);
  virtual ~U4TPCHit();

  inline virtual Int_t     GetPadRowNumber() const { return fPadRowNumber; }
  inline virtual Double_t  GetPadRowRadius() const { return fPadRowRadius; }
  inline virtual Int_t     GetSide        () const { return fSide;         }

  inline virtual Bool_t    IsDone         () const { return fDone;         }

  inline virtual Double_t  GetRPhi   () { if (!fDone) Update(); return fRPhi;    }
  inline virtual Double_t  GetDRPhi  () { if (!fDone) Update(); return fDRPhi;   }
  inline virtual Double_t  GetZdrift () { if (!fDone) Update(); return fZdrift;  }
  inline virtual Double_t  GetDZdrift() { if (!fDone) Update(); return fDZdrift; }
  inline virtual Double_t  GetVdrift () { if (!fDone) Update(); return fVdrift;  }
  inline virtual Double_t  GetPhi    () { if (!fDone) Update(); return fPhi;     }
  
  inline virtual void      SetRPhi   (Double_t d)  { fRPhi    = d; }
  inline virtual void      SetDRPhi  (Double_t d)  { fDRPhi   = d; }
  inline virtual void      SetZdrift (Double_t d)  { fZdrift  = d; }
  inline virtual void      SetDZdrift(Double_t d)  { fDZdrift = d; }
  inline virtual void      SetPhi    (Double_t d)  { fPhi     = d; }
  inline virtual void      SetVdrift (Double_t d)  { fVdrift  = d; }

  inline virtual void      SetDone   (Bool_t   b)  { fDone    = b; }

  virtual Bool_t           IsSortable()                const;
  virtual Int_t            Compare(const TObject *obj) const;

  virtual void             DebugPrint(const Char_t *opt = "Brief");

protected:
  virtual void Update();
private:
  virtual void CalculatePosition();

private:
  Bool_t	fDone;                // (1,0) = (cluster done, not yet)
  Double_t	fRPhi;                // drift distance (cm)
  Double_t	fDRPhi;               // error on drift distance (cm)
  Double_t	fZdrift;              // drift distance (cm)
  Double_t	fDZdrift;             // error on drift distance (cm)

  Int_t         fPadRowNumber;        // pad row number
  Double_t      fPadRowRadius;        // pad row radius
  Int_t         fSide;                // (-1, +1) = (-z, +z)
  Double_t      fVdrift;              // drift velocity
  Double_t      fPhi;                 // phi


  ClassDef(U4TPCHit, 1)            // TPC Anode Hit
};

#endif
